c
c-----------------------------------------------------------------------
c
      subroutine cubicate (level, phaseth, phaseph, uout)
c
c.....cubicate - tessellation of the sphere based on an iterative
c     division of a cube.
c
      include           'implicit.inc'
c
      integer           level, uout
      real*8            phaseth, phaseph
c
c.....Polyhedra data structure:
c
      include           'polyhedron.inc'
c
      integer           ix(mvert), iy(mvert), iz(mvert) !vert. coord.
      integer           ilev, i, j, nf1
      integer           i1, i2, i3, i4, i5, i6, i7, i8, i9
      integer           ix5, iy5, iz5, ix6, iy6, iz6, ix7, iy7, iz7
      integer           ix8, iy8, iz8, ix9, iy9, iz9
      integer           nvertold
      logical           new, project
      real*8            xpt, ypt, zpt
      real*8            rr
      real*8            theta, phi, stheta
c
      if (level .lt. 0) then
         project = .true.
         level = -level
      else
         project = .false.
      endif
c
c.....First, let us compute a convex polyhedra that approximately
c     cubicates the sphere. We will start with a cube and
c     succesively divide each quadrilateral facet into four new ones.
c     The coordinates for the vertices will be maintained as integer
c     values to get best precision and efficiency.
c
      nvf = 4         !Each face will be a quadrilateral
c
      nvert = 8
      ix(1) =  1024
      iy(1) =  1024
      iz(1) =  1024
c
      ix(2) = -1024
      iy(2) =  1024
      iz(2) =  1024
c
      ix(3) = -1024
      iy(3) = -1024
      iz(3) =  1024
c
      ix(4) =  1024
      iy(4) = -1024
      iz(4) =  1024
c
      ix(5) =  1024
      iy(5) =  1024
      iz(5) = -1024
c
      ix(6) = -1024
      iy(6) =  1024
      iz(6) = -1024
c
      ix(7) = -1024
      iy(7) = -1024
      iz(7) = -1024
c
      ix(8) =  1024
      iy(8) = -1024
      iz(8) = -1024
c
c.....store initial faces of the octahedron:
c
      isw0 = 1
      isw1 = 0
      nface(0) = 6
      iface(1,1,0) = 1
      iface(2,1,0) = 5
      iface(3,1,0) = 6
      iface(4,1,0) = 2
c
      iface(1,2,0) = 2
      iface(2,2,0) = 6
      iface(3,2,0) = 7
      iface(4,2,0) = 3
c
      iface(1,3,0) = 3
      iface(2,3,0) = 7
      iface(3,3,0) = 8
      iface(4,3,0) = 4
c
      iface(1,4,0) = 4
      iface(2,4,0) = 8
      iface(3,4,0) = 5
      iface(4,4,0) = 1
c
      iface(1,5,0) = 3
      iface(2,5,0) = 4
      iface(3,5,0) = 1
      iface(4,5,0) = 2
c
      iface(1,6,0) = 8
      iface(2,6,0) = 7
      iface(3,6,0) = 6
      iface(4,6,0) = 5
c
c.....In PROJECT mode get cartesian coordinates projected out to the
c     surface of the sphere:
c
      if (project) then
         do j = 1, nvert
            xpt = dble(ix(j))
            ypt = dble(iy(j))
            zpt = dble(iz(j))
            rr = sqrt(xpt*xpt + ypt*ypt + zpt*zpt)
            theta = acos(zpt/rr) + phaseth
            phi = atan2(ypt,xpt) + phaseph
            stheta = sin(theta)
            sthcph(j) = stheta * cos(phi)
            sthsph(j) = stheta * sin(phi)
            cth(j) = cos(theta)
            xvert(j) = radius * sthcph(j)
            yvert(j) = radius * sthsph(j)
            zvert(j) = radius * cth(j)
         enddo
      endif
c
      do ilev = 1, min(level,mlevel)
c
c........exchange bins for old and new levels:
c
         nvertold = nvert
         isw0 = mod(isw0+1,2)
         isw1 = mod(isw0+1,2)
         nface(isw1) = 0
c
c........iterate over the facets of the old level:
c
         do i = 1, nface(isw0)
            i1 = iface(1,i,isw0)
            i2 = iface(2,i,isw0)
            i3 = iface(3,i,isw0)
            i4 = iface(4,i,isw0)
c
c...........get new points to divide this triangle:
c
            ix5 = (ix(i1)+ix(i2))/2
            iy5 = (iy(i1)+iy(i2))/2
            iz5 = (iz(i1)+iz(i2))/2
            ix6 = (ix(i2)+ix(i3))/2
            iy6 = (iy(i2)+iy(i3))/2
            iz6 = (iz(i2)+iz(i3))/2
            ix7 = (ix(i3)+ix(i4))/2
            iy7 = (iy(i3)+iy(i4))/2
            iz7 = (iz(i3)+iz(i4))/2
            ix8 = (ix(i4)+ix(i1))/2
            iy8 = (iy(i4)+iy(i1))/2
            iz8 = (iz(i4)+iz(i1))/2
            ix9 = (ix(i1)+ix(i2)+ix(i3)+ix(i4))/4
            iy9 = (iy(i1)+iy(i2)+iy(i3)+iy(i4))/4
            iz9 = (iz(i1)+iz(i2)+iz(i3)+iz(i4))/4
c
c...........store only new points:
c
            new = .true.
            i5 = 0
            do while (new .and. i5.lt.nvert)
               i5 = i5 + 1
               if (ix(i5) .eq. ix5) then
                  if (iy(i5) .eq. iy5) then
                     if (iz(i5) .eq. iz5) then
                        new = .false.
                     endif
                  endif
               endif
            enddo
            if (new) then
               nvert = nvert + 1
               i5 = nvert
               ix(i5) = ix5
               iy(i5) = iy5
               iz(i5) = iz5
               if (project) then
                  xvert(i5) = 0.5d0 * (xvert(i1)+xvert(i2))
                  yvert(i5) = 0.5d0 * (yvert(i1)+yvert(i2))
                  zvert(i5) = 0.5d0 * (zvert(i1)+zvert(i2))
               endif
            endif
c
            new = .true.
            i6 = 0
            do while (new .and. i6.lt.nvert)
               i6 = i6 + 1
               if (ix(i6) .eq. ix6) then
                  if (iy(i6) .eq. iy6) then
                     if (iz(i6) .eq. iz6) then
                        new = .false.
                     endif
                  endif
               endif
            enddo
            if (new) then
               nvert = nvert + 1
               i6 = nvert
               ix(i6) = ix6
               iy(i6) = iy6
               iz(i6) = iz6
               if (project) then
                  xvert(i6) = 0.5d0 * (xvert(i2)+xvert(i3))
                  yvert(i6) = 0.5d0 * (yvert(i2)+yvert(i3))
                  zvert(i6) = 0.5d0 * (zvert(i2)+zvert(i3))
               endif
            endif
c
            new = .true.
            i7 = 0
            do while (new .and. i7.lt.nvert)
               i7 = i7 + 1
               if (ix(i7) .eq. ix7) then
                  if (iy(i7) .eq. iy7) then
                     if (iz(i7) .eq. iz7) then
                        new = .false.
                     endif
                  endif
               endif
            enddo
            if (new) then
               nvert = nvert + 1
               i7 = nvert
               ix(i7) = ix7
               iy(i7) = iy7
               iz(i7) = iz7
               if (project) then
                  xvert(i7) = 0.5d0 * (xvert(i3)+xvert(i4))
                  yvert(i7) = 0.5d0 * (yvert(i3)+yvert(i4))
                  zvert(i7) = 0.5d0 * (zvert(i3)+zvert(i4))
               endif
            endif
c
            new = .true.
            i8 = 0
            do while (new .and. i8.lt.nvert)
               i8 = i8 + 1
               if (ix(i8) .eq. ix8) then
                  if (iy(i8) .eq. iy8) then
                     if (iz(i8) .eq. iz8) then
                        new = .false.
                     endif
                  endif
               endif
            enddo
            if (new) then
               nvert = nvert + 1
               i8 = nvert
               ix(i8) = ix8
               iy(i8) = iy8
               iz(i8) = iz8
               if (project) then
                  xvert(i8) = 0.5d0 * (xvert(i4)+xvert(i1))
                  yvert(i8) = 0.5d0 * (yvert(i4)+yvert(i1))
                  zvert(i8) = 0.5d0 * (zvert(i4)+zvert(i1))
               endif
            endif
c
            new = .true.
            i9 = 0
            do while (new .and. i9.lt.nvert)
               i9 = i9 + 1
               if (ix(i9) .eq. ix9) then
                  if (iy(i9) .eq. iy9) then
                     if (iz(i9) .eq. iz9) then
                        new = .false.
                     endif
                  endif
               endif
            enddo
            if (new) then
               nvert = nvert + 1
               i9 = nvert
               ix(i9) = ix9
               iy(i9) = iy9
               iz(i9) = iz9
               if (project) then
                  xvert(i9) = 0.25d0 *
     &                        (xvert(i1)+xvert(i2)+xvert(i3)+xvert(i4))
                  yvert(i9) = 0.25d0 *
     &                        (yvert(i1)+yvert(i2)+yvert(i3)+yvert(i4))
                  zvert(i9) = 0.25d0 *
     &                        (zvert(i1)+zvert(i2)+zvert(i3)+zvert(i4))
               endif
            endif
c
c...........store the four new faces that result from the division of
c           the old face:
c
            nf1 = nface(isw1)
            iface(1,nf1+1,isw1) = i1
            iface(2,nf1+1,isw1) = i5
            iface(3,nf1+1,isw1) = i9
            iface(4,nf1+1,isw1) = i8
c
            iface(1,nf1+2,isw1) = i8
            iface(2,nf1+2,isw1) = i9
            iface(3,nf1+2,isw1) = i7
            iface(4,nf1+2,isw1) = i4
c
            iface(1,nf1+3,isw1) = i5
            iface(2,nf1+3,isw1) = i2
            iface(3,nf1+3,isw1) = i6
            iface(4,nf1+3,isw1) = i9
c
            iface(1,nf1+4,isw1) = i9
            iface(2,nf1+4,isw1) = i6
            iface(3,nf1+4,isw1) = i3
            iface(4,nf1+4,isw1) = i7
            nface(isw1) = nf1 + 4
         enddo
c
c........In PROJECT mode project out the new points on the sphere
c        surface:
c
         if (project) then
            do j = nvertold+1, nvert
               xpt = xvert(j)
               ypt = yvert(j)
               zpt = zvert(j)
               rr = sqrt(xpt*xpt + ypt*ypt + zpt*zpt)
               theta = acos(zpt/rr) + phaseth
               phi = atan2(ypt,xpt) + phaseph
               stheta = sin(theta)
               sthcph(j) = stheta * cos(phi)
               sthsph(j) = stheta * sin(phi)
               cth(j) = cos(theta)
               xvert(j) = radius * sthcph(j)
               yvert(j) = radius * sthsph(j)
               zvert(j) = radius * cth(j)
            enddo
         endif
c
      enddo
c
c.....The final set of faces is in the isw1 bin.
c.....Determine the spherical coordinates of the vertices, and store
c     sin(theta)sin(phi), sin(theta)cos(phi), and cos(theta):
c
      if (project) return
      do j = 1, nvert
         xpt = dble(ix(j))
         ypt = dble(iy(j))
         zpt = dble(iz(j))
         rr = sqrt(xpt*xpt + ypt*ypt + zpt*zpt)
         theta = acos(zpt/rr) + phaseth
         phi = atan2(ypt,xpt) + phaseph
         stheta = sin(theta)
         sthcph(j) = stheta * cos(phi)
         sthsph(j) = stheta * sin(phi)
         cth(j) = cos(theta)
      enddo
c
c.....Determine cartesian coordinates of the points in the surface of the
c     sphere:
c
      do j = 1, nvert
         xvert(j) = radius * sthcph(j)
         yvert(j) = radius * sthsph(j)
         zvert(j) = radius * cth(j)
      enddo
c
      write (uout,100) level, phaseth, phaseph
      write (uout,105) nvert, nface(isw1)
 100  format (/
     & 1x, 'CUBICATE: Tesselation of the sphere by iterative division '/
     & 1x, '          of the faces of a cube.'//
     & 1x, 'Iteration level: ', i4/
     & 1x, 'Phase shift for theta coordinate: ', 1p, e12.4, 0p/
     & 1x, 'Phase shift for phi   coordinate: ', 1p, e12.4, 0p )
 105  format (
     & 1x, 'Number of vertices created:    ', i6/
     & 1x, 'Number of quadrilateral faces: ', i6)
c
      end
