c
c-----------------------------------------------------------------------
c
      subroutine molarrow (
     &  atom1, atom2,  !Name of the atoms to be connected
     &  dmin, dmax,    !Limit distances otherwise
     &  stickrad,      !Stick radius
     &  conerad,       !Cone base radius
     &  conelength,    !Cone length
     &  usedefrgb,     !Use default colors?
     &  rgb,           !Use this color instead
     &  itexture,      !POV texture index
     &  uout)
c
c.....molarrow - create an arrow from those atom1 to atom2 positions.
c
      include           'implicit.inc'
      include           'consts.inc'
      include           'datatm.inc'
      include           'error.inc'
      include           'molec.inc'
      include           'molge.inc'
c
      character*(*)     atom1, atom2
      integer           uout, itexture
      real*8            dmin, dmax, stickrad, conerad, conelength
      real*8            rgb(4)
      logical           usedefrgb
c
      integer           i, i0, i1, i2, j
      real*8            defdmin, defdmax, defrgb(4)
      real*8            xx, yy, zz, d2, x1, y1, z1, x2, y2, z2, t, r
      logical           equal, is1, is2, doit
c
      i0 = gnpt
      i1 = gnstk
      i2 = gncone
c
      defdmin = dmin*dmin
      defdmax = dmax*dmax
      defrgb(1) = rgb(1)
      defrgb(2) = rgb(2)
      defrgb(3) = rgb(3)
      defrgb(4) = rgb(4)
c
c.....Setting default colors:
c
      if (usedefrgb) then         !Use brown color for the bonds
         defrgb(1) = 0.647059d0
         defrgb(2) = 0.164706d0
         defrgb(3) = 0.164706d0
         defrgb(4) = 0.0d0
      endif
c
c.....Search for the adequate atom types:
c
      do i = 1, nmol-1
         if (codemol(i).eq.codecrystal) then
            is1 = equal(namneq(typmol(i)),atom1)
            is2 = equal(namneq(typmol(i)),atom2)
         else
            is1 = equal(namclus(typmol(i)),atom1)
            is2 = equal(namclus(typmol(i)),atom2)
         endif
         do j = i+1, nmol
            if (codemol(j).eq.codecrystal) then
               doit = (is1 .and. equal(namneq(typmol(j)),atom2)) .or.
     &                (is2 .and. equal(namneq(typmol(j)),atom1))
            else
               doit = (is1 .and. equal(namclus(typmol(j)),atom2)) .or.
     &                (is2 .and. equal(namclus(typmol(j)),atom1))
            endif
c
c...........One of the atoms, at least, must be inside the graphical
c           clipping box:
c
            doit = doit .and. (inclip(i) .or. inclip(j))
c
c...........Analyze if the atom distances are in the right range:
c
            if (doit) then
               xx = (xmol(i)-xmol(j)) * ared
               yy = (ymol(i)-ymol(j)) * bred
               zz = (zmol(i)-zmol(j)) * cred
               d2 = xx*xx + yy*yy + zz*zz + 2d0 *
     &              (xx*yy*cgamm + yy*zz*calph + zz*xx*cbeta)
               doit = (d2.ge.defdmin) .and. (d2.le.defdmax)
            endif
c
c...........If all requeriments are fulfilled add the new stick:
c
            if (doit) then
               r = sqrt(d2)
               t = 1d0 - min(0.5d0, conelength/r)
*mdc*if debug
*               write (uout,*) 'DBG(molarrow) cnl,r,t:',
*     &                        conelength, r, t
*mdc*endif
               if (gnpt+3.gt.GMPT) call error ('molarrow',
     &            'Too many points. Increase GMPT', faterr)
               if (gnstk+1.gt.GMSTK) call error ('molarrow',
     &            'Too many sticks. Increase GMSTK', faterr)
               if (gncone+1.gt.GMCONE) call error ('molarrow',
     &            'Too many cones. Increase GMCONE', faterr)
               gnstk = gnstk + 1
               gstkpt(1,gnstk) = gnpt + 1
               gstkpt(2,gnstk) = gnpt + 2
               gstkrad(gnstk) = stickrad
               gstkdefrgb(gnstk) = usedefrgb
               gstkrgbf(1,gnstk) = defrgb(1)
               gstkrgbf(2,gnstk) = defrgb(2)
               gstkrgbf(3,gnstk) = defrgb(3)
               gstkrgbf(4,gnstk) = defrgb(4)
               gstktext(gnstk) = itexture
               gncone = gncone + 1
               gcnpt(1,gncone) = gnpt + 2
               gcnpt(2,gncone) = gnpt + 3
               gcnrad(gncone) = conerad
               gcndefrgb(gncone) = usedefrgb
               gcnrgbf(1,gncone) = defrgb(1)
               gcnrgbf(2,gncone) = defrgb(2)
               gcnrgbf(3,gncone) = defrgb(3)
               gcnrgbf(4,gncone) = defrgb(4)
               gcntext(gncone) = itexture
               if (is1) then
                  x1 = xmol(i)
                  y1 = ymol(i)
                  z1 = zmol(i)
                  x2 = xmol(j)
                  y2 = ymol(j)
                  z2 = zmol(j)
               else
                  x1 = xmol(j)
                  y1 = ymol(j)
                  z1 = zmol(j)
                  x2 = xmol(i)
                  y2 = ymol(i)
                  z2 = zmol(i)
               endif
               xx = x1 + (x2-x1) * t
               yy = y1 + (y2-y1) * t
               zz = z1 + (z2-z1) * t
               gnpt = gnpt + 1
               gxpt(gnpt) = x1
               gypt(gnpt) = y1
               gzpt(gnpt) = z1
               gnpt = gnpt + 1
               gxpt(gnpt) = xx
               gypt(gnpt) = yy
               gzpt(gnpt) = zz
               gnpt = gnpt + 1
               gxpt(gnpt) = x2
               gypt(gnpt) = y2
               gzpt(gnpt) = z2
            endif
         enddo
      enddo
c
c.....transform points to cartesian coordinates:
c
      do i = i0 + 1, gnpt
         gxpt(i) = gxpt(i)*crys2car(1) + gypt(i)*crys2car(2) +
     &             gzpt(i)*crys2car(3)
         gypt(i) = gypt(i)*crys2car(4) + gzpt(i)*crys2car(5)
         gzpt(i) = gzpt(i)*crys2car(6)
      enddo
c
      if (uout.gt.0) write (uout,810) gnstk-i1, gncone-i2
c
c.....FORMATS:
c
 810  format (
     & 1x, 'MOLARROW: ', i6, ' sticks and', i6,
     & ' cones added to the molecular graphics database.')
c
      end
