c
c-----------------------------------------------------------------------
c
      subroutine molballs (
     &  atom,       !Name of the atom to be put as ball
     &  usedefrad,  !Determine a default radius?
     &  ballrad,    !Radius to be used instead
     &  usedefrgb,  !Use default colors?
     &  rgb,        !Use this color instead
     &  itexture,   !POV texture index
     &  imtype,     !Add an image map on this balls
     &  imstart,    !Pointer to the first char of the image map file
     &  uout)
c
c.....molballs - Transform an atom type into balls.
c
      include           'implicit.inc'
      include           'strtools.inc'
      include           'consts.inc'
      include           'datatm.inc'
      include           'error.inc'
      include           'molec.inc'
      include           'molge.inc'
c
      character*(*)     atom
      integer           uout, imtype, imstart, itexture
      real*8            ballrad, rgb(4)
      logical           usedefrad, usedefrgb
c
      integer           i, i0, j, atmZ
      real*8            qqq, defradius, defrgb(4)
      character*(3)     name
      character*(6)     uppcase
      logical           equal, doit
      integer           zhlookup
c
      i0 = gnpt
c
c.....Setting of default radii and colors if required:
c
      if (usedefrad .or. usedefrgb) then
         name = atom
         name = uppcase(name)
         if (name(2:2).lt.'A' .or. name(2:2).gt.'Z') then
            name(2:2) = ' '
         endif
         name(3:3) = null
         atmZ = zhlookup (name)
         if (atmZ .le. 0) call error ('molballs',
     &      'Atom name not recognized in BALLS order!', faterr)
*mdc*if debug
*         write (uout,900) name(1:2), atmZ
*mdc*endif
      endif
      if (usedefrad) then
         if (typrad .eq. typION) then
            defradius = (atmION(0,atmZ)/bohr2pm) * grfactor
         else if (typrad .eq. typvdW) then
            defradius = (atmvdW(atmZ)/bohr2pm) * grfactor
         else if (typrad .eq. typcov) then
            defradius = (atmcov(atmZ)/bohr2pm) * grfactor
         endif
      else
         defradius = ballrad
      endif
      if (usedefrgb) then
         defrgb(1) = dble(CPKcol(1,atmCPK(atmZ))) / 255d0
         defrgb(2) = dble(CPKcol(2,atmCPK(atmZ))) / 255d0
         defrgb(3) = dble(CPKcol(3,atmCPK(atmZ))) / 255d0
         defrgb(4) = 0d0
      else
         defrgb(1) = rgb(1)
         defrgb(2) = rgb(2)
         defrgb(3) = rgb(3)
         defrgb(4) = rgb(4)
      endif
c
c.....Search for the atom type among non-equivalent crystal atoms:
c
      do j = 1, nmol
         if (inclip(j)) then
            doit = .false.
            if (codemol(j).eq.codecrystal) then
               doit = equal(namneq(typmol(j)),atom)
               if (doit) qqq = qneq(typmol(j))
            else
               doit = equal(namclus(typmol(j)),atom)
               if (doit) qqq = qclus(typmol(j))
            endif
            if (doit) then
               if (usedefrad .and. (typrad .eq. typION))
     &            defradius = (atmION(nint(qqq),atmZ)/bohr2pm)
     &                        * grfactor
               gnpt = gnpt + 1
               if (gnpt.gt.GMPT) call error ('molballs',
     &            'Too many points. Increase GMPT', faterr)
               gxpt(gnpt) = xmol(j)
               gypt(gnpt) = ymol(j)
               gzpt(gnpt) = zmol(j)
               gnball = gnball + 1
               if (gnball.gt.GMBALL) call error ('molballs',
     &            'Too many balls. Increase GMBALL', faterr)
               gballct(gnball)      = gnpt
               gballrad(gnball)     = defradius
               gballrgbf(1,gnball)  = defrgb(1)
               gballrgbf(2,gnball)  = defrgb(2)
               gballrgbf(3,gnball)  = defrgb(3)
               gballrgbf(4,gnball)  = defrgb(4)
               gballtext(gnball)    = itexture
               gballmaptype(gnball) = imtype
               gballfp(gnball)      = imstart
            endif
         endif
      enddo
c
c.....transform points to cartesian coordinates:
c
      do i = i0 + 1, gnpt
         gxpt(i) = gxpt(i)*crys2car(1) + gypt(i)*crys2car(2) +
     &             gzpt(i)*crys2car(3)
         gypt(i) = gypt(i)*crys2car(4) + gzpt(i)*crys2car(5)
         gzpt(i) = gzpt(i)*crys2car(6)
      enddo
c
      if (uout.gt.0) write (uout,810) gnpt-i0
c
c.....FORMATS:
c
 810  format (
     & 1x, 'MOLBALLS: ', i6,
     & ' spheres added to the molecular graphics database.')
*mdc*if debug
* 900  format (1x, 'DBG(molballs.1). Atom:', a, i5)
*mdc*endif
c
      end
