c
c.....molec.inc - database of molecular structure.
c
c.....The Molecular model (I)
c
c The molecular model is defined as:
c       CRYSTAL + CLUSTER - HOLE
c and any of the three components may be empty. Coordinates
c are crystallographic by default, i.e. they are given relative
c to the unit cell axes.
c
c.....CRYSTAL data:
c
c spg ......... space group
c ared-cred ... unit cell side lengths
c alph-gamm ... unit cell angles (DEGREES)
c calph-cgamm . angle cosines
c salph-sgamm . angle sines
c g() ......... metric tensor
c crys2car() .. matrix used to transform crystallographic coordinates
c               into orthogonal cartesian coordinates
c MNEQ ........ max. number of symmetry different atoms in the unit cell
c nneq ........ actual number of symmetry diff. atoms
c xneq-zneq ... crystallographic coordinates of the symmetry diff. atoms
c qneq ........ charge of the atom
c namneq ...... name of the atom
c multneq ..... multiplicity, i.e. number of identical atoms in the unit
c               cell
c zzneq ....... atomic number of this type of atom
c MCELL ....... max. number of atoms in the unit cell
c ncell ....... actual number of atoms
c xcell-zcell . crystallographic coordinates of the atoms
c typcell ..... type of atom (among those diff. by symmetry)
c
      integer           MNEQ, MCELL
      parameter         (MNEQ=40, MCELL=600)
      character*(40)    spg
      integer           nneq, ncell, multneq(MNEQ), zzneq(MNEQ)
      real*8            ared, bred, cred, alph, beta, gamm
      real*8            calph, cbeta, cgamm
      real*8            salph, sbeta, sgamm
      real*8            g(6), crys2car(6)
      real*8            xneq(MNEQ), yneq(MNEQ), zneq(MNEQ), qneq(MNEQ)
      character*(6)     namneq(MNEQ)
      real*8            xcell(MCELL), ycell(MCELL), zcell(MCELL)
      integer           typcell(MCELL)
c
      common /ms1s/ spg, namneq
      common /ms1i/ nneq, ncell, multneq, zzneq, typcell
      common /ms1r/ ared, bred, cred, alph, beta, gamm,
     &              calph, cbeta, cgamm,
     &              salph, sbeta, sgamm,
     &              g, crys2car,
     &              xneq, yneq, zneq, qneq,
     &              xcell, ycell, zcell
c
c.....* NOTICE * This is a dirty trick to adapt the Ewald routines
c     The same ared..cred, etc variables will be given two different
c     names:
c
      real*8            aa(3), bb(3), cosdir(3), sindir(3)
      equivalence       (aa(1), ared), (aa(2), bred), (aa(3), cred)
      equivalence       (bb(1), alph), (bb(2), beta), (bb(3), gamm)
      equivalence       (cosdir(1), calph), (cosdir(2), cbeta),
     &                  (cosdir(3), cgamm)
      equivalence       (sindir(1), salph), (sindir(2), sbeta),
     &                  (sindir(3), sgamm)
c
c.....Primitive and reciprocal lattice parameters:
c
c arec-crec ..... reciprocal lattice parameters
c calphr-cgammr . cosines of the reciprocal lattice angles
c gr() .......... metric tensor of the reciprocal lattice
c gp(,) ......... metric tensor of the primitive real lattice
c grp(,) ........ metric tensor of the primitive reciprocal lattice
c
      real*8            arec, brec, crec, calphr, cbetar, cgammr,
     &                  gr(6), gp(3,3), grp(3,3)
      common /ms2r/ arec, brec, crec, calphr, cbetar, cgammr, gr,
     &              gp, grp
c
c.....HOLE and CLUSTER structures:
c
c MHOLE ....... Max. number of crystal positions that can be declared
c               empty (i.e. holes)
c nhole ....... actual number of holes
c xhole-zhole . crystallographic coordinates of the hole positions
c MNEQCLUS .... Max. number of symmetry different cluster atoms
c nneqclus .... actual number of symmetry different cluster atoms
c multclus .... number of identical atoms
c qclus ....... charge of this type of atoms
c namclus ..... name of symmetry different cluster atoms
c MCLUS ....... Max. number of cluster atoms
c nclus ....... actual number of cluster atoms
c xclus-zclus . crsyallographic coordinates of the cluster atoms
c typclus ..... type of cluster atom (among those diff. by symmetry)
c zzclus....... atomic number of this type of atom
c
      integer           MHOLE, MNEQCLUS, MCLUS
      parameter         (MHOLE=300, MNEQCLUS=2000, MCLUS=2000)
      integer           nhole, nneqclus, nclus
      real*8            xhole(MHOLE), yhole(MHOLE), zhole(MHOLE)
      integer           multclus(MNEQCLUS), zzclus(MNEQCLUS)
      real*8            qclus(MNEQCLUS)
      character*(6)     namclus(MNEQCLUS)
      real*8            xclus(MCLUS), yclus(MCLUS), zclus(MCLUS)
      integer           typclus(MCLUS)
c
      common /ms3s/ namclus
      common /ms3i/ nhole, nneqclus, nclus, multclus, zzclus, typclus
      common /ms3r/ xhole, yhole, zhole, xclus, yclus, zclus, qclus
c
c.....CLIPPING boxes structures:
c
c Two clipping boxes may be defined, each of them being a parallelepiped
c parallel to the unit cell:
c
c a) The MOLECULAR clipping box determines the number of atoms of the
c    molecular model that will be used to build the molecular plot.
c b) The GRAPHICAL clipping determines the atoms that will be actually
c    displayed.
c
c The molecular box MUST be equal or larger than the graphical box.
c
      real*8            x0molclip, x1molclip, y0molclip, y1molclip,
     &                  z0molclip, z1molclip
      real*8            x0grfclip, x1grfclip, y0grfclip, y1grfclip,
     &                  z0grfclip, z1grfclip
c
      common /msclip/   x0molclip, x1molclip, y0molclip, y1molclip,
     &                  z0molclip, z1molclip,
     &                  x0grfclip, x1grfclip, y0grfclip, y1grfclip,
     &                  z0grfclip, z1grfclip
c
c
c.....The Molecular model (II)
c     Composed by atoms of the crystal or the cluster, the molecular
c     model will contain all atoms that may appear on the scene.
c
c.....molecular model structure:
c
c MMOL ....... max. number of atoms in the molecular model
c nmol ....... actual number of atoms
c xmol-zmol .. cartesian coordinates (maybe crystallographic or
c              real orthogonal cartesian, depending on the program
c              status)
c inclip ..... is this atom inside the graphical box ?
c isball ..... will this atom be represented as a ball ?
c codemol .... is this atom part of the crystal or part of the cluster
c typmol ..... type of cluster or crystal atom
c
      integer           MMOL
c      parameter         (MMOL = 3*(MCELL+MCLUS))
      parameter         (MMOL = 3000)
      integer           codecrystal, codecluster
      parameter         (codecrystal=101, codecluster=102)
      integer           nmol, codemol(MMOL), typmol(MMOL)
      real*8            xmol(MMOL), ymol(MMOL), zmol(MMOL)
      logical           inclip(MMOL), isball(MMOL)
c
      common /ms4i/ nmol, codemol, typmol
      common /ms4r/ xmol, ymol, zmol
      common /ms4l/ inclip, isball
