c
c-----------------------------------------------------------------------
c
      subroutine molenv (atom, xct, yct, zct, nshells, prtlevel, uout)
c
c.....molenv - build succesive shells around the target. Each shell is
c     formed by all the identical atoms equidistant to the target.
c     This information is reported to the user.
c
c GLOBAL DATA:
c
      include           'implicit.inc'
      include           'datatm.inc'
      include           'error.inc'
      include           'molec.inc'
      include           'molge.inc'
      include           'strtools.inc'
      include           'consts.inc'
c
c PARAMETERS:
c
      character*(6)     atom          ! Atom name
      integer           nshells       ! Number of shells to be printed
      integer           prtlevel      ! How much to print
      integer           uout          ! Print logical unit
      real*8            xct, yct, zct ! Xtal coordinates of the point
c
c.....local variables and functions:
c
      integer           MENV
      parameter         (MENV = MMOL)
      integer           nenv, isort(MENV), typenv(MENV)
      real*8            renv(MENV)
      integer           MSHELL
      parameter         (MSHELL = 100)
      integer           nsh
*      integer           nfish(MSHELL)
      real*8            rsh(MSHELL)
      integer           it(MSHELL), natsh(MSHELL)
      character*(6)     namsh(MSHELL)
      real*8            rr, xx1, yy1, zz1
      integer           il, first, last, nshmax, tt
      integer           i, k
      logical           mrat
      integer           leng
c
      real*8            dis2, xi, yi, zi
      dis2(xi,yi,zi) = xi * (xi*g(1) + two*yi*g(2) + two*zi*g(4)) +
     &                 yi * (yi*g(3) + two*zi*g(5)) + zi*zi*g(6)
c
c.....run over the atoms in the molecular model, and determine the
c     distance to the target:
c
      nenv = nmol
      do i = 1, nmol
         xx1 = xmol(i) - xct
         yy1 = ymol(i) - yct
         zz1 = zmol(i) - zct
         isort(i) = i
         renv(i) = sqrt(dis2(xx1,yy1,zz1))
         if (renv(i) .le. eps) renv(i) = 1d20
         typenv(i) = typmol(i)
         if (codemol(i) .eq. codecrystal) typenv(i) = -typmol(i)
      enddo
c
c.....sort the neighbors by distance:
c
      call qcksort (renv, isort, 1, nenv)
c
c.....classify by type the ions at the same distance:
c
      first = 1
      last  = 1
      rr = renv(isort(first))
      do while (last .le. nenv)
         mrat = last.lt.nenv
         if (mrat) mrat = abs(renv(isort(last+1)) - rr) .le. eps
         if (mrat) then
            last = last + 1
         else
            call iqcksort (typenv, isort, first, last)
            last = last + 1
            first = last
            if (last.lt.nenv) rr = renv(isort(last))
         endif
      enddo
c
c.....sort out and print the shells:
c
      nshmax = nshells
      if (nsh .ge. MSHELL) then
         call error ('molenv', 'Too many shells. (MSHELL)'//
     &               ' Trying our max. capacity', warning)
         nshmax = MSHELL
      endif
      if (prtlevel .gt. 0)
     &    write (uout,300) atom(1:leng(atom)), xct, yct, zct
      first = 1
      last  = 1
      rr = renv(isort(first))
      tt = typenv(isort(first))
      nsh = 0
      do while (last.le.nenv .and. nsh.le.nshmax)
         mrat = last .lt. nenv
         il = isort(last+1)
         if (mrat) mrat = abs(renv(il)-rr).le.eps .and. typenv(il).eq.tt
         if (mrat) then
            last = last + 1
         else
            nsh = nsh + 1
*            nfish(nsh) = first
            rsh(nsh) = rr
            it(nsh) = typenv(isort(first))
            natsh(nsh) = last - first + 1
            if (it(nsh) .lt. 0) then
               namsh(nsh) = namneq(-it(nsh))
            else
               namsh(nsh) = namclus(it(nsh))
            endif
c
c...........Print out long description of neighbor shells:
c
            if (prtlevel .gt. 0) then
               write (uout,310) nsh, rr, last-first+1
               write (uout,320) (namsh(nsh)(1:leng(namsh(nsh))),
     &                it(nsh),
     &                xmol(isort(k)), ymol(isort(k)), zmol(isort(k)),
     &                k = first, last)
            endif
            last = last + 1
            first = last
            if (first.lt.nenv) then
               rr = renv(isort(first))
               tt = typenv(isort(first))
            endif
         endif
         if (nshmax.gt.0 .and. nsh.eq.nshmax) goto 101 
      enddo
c
 101  continue
c
c.....Print out short description of the environment:
c
      write (uout,650) atom(1:leng(atom)), xct, yct, zct
      write (uout,662) nsh,
     &      (natsh(k), namsh(k)(1:leng(namsh(k))), it(k), rsh(k),
     &      k = 1, nsh)
c
      return
c
c.....formats:
c
 300  format (/1x, 'ENVIRONMENT LONG DESCRIPTION :'/
     & 1x,'Local environment of ', a,' situated at ',3f10.5)
 310  format (/1x, 'Shell #', i3, ' at distance', 1pd15.8, ' formed by',
     & i4, ' atoms')
 320  format (2 (1x, '(', a4, i3, 3f9.5, ')') )
 650  format (/1x, 'ENVIRONMENT SHORT DESCRIPTION :'/
     & 1x, 'Active ion: ', a, '  Position: ', 3f10.5)
 655  format (1x, 'Title: ', a)
 662  format (1x, 'Number of shells in environment: ', i3/
     & 1x, 'List of shells --> (number_of_ions name (type) distance)'/
     & ( 2(1x,'(',i3,1x,a4,' (',i3,')',f15.6,')',2x) ) )
      end
