c
c.....molge.inc - database of graphical elements forming the molecular
c                 plot.
c
c The graphical elements are: POINTS, BALLS, STICKS, POLYGONS and
c POLYHEDRA.
c
c GMPT .......... max. number of points
c gnpt .......... actual number of points
c gxpt-gzpt ..... cartesian coordinates of point
c
c GMBALL ........ max number of balls
c gnball ........ actual number of balls
c gballrad ...... radius for the ball
c gballrgbf ..... <red,green,blue,filter> color of the ball
c gballct ....... index to the point in which the ball is centered
c gballtext ..... POV-Ray texture index
c 
c GMSTK ......... max. number of sticks
c gnstk ......... actual number of sticks
c gstkpt ........ index to the points linked by this stick
c gstkrad ....... stick radius
c gstkdefrbg .... Use default texture for sticks?
c gstkrgbf ...... <r,g,b,f> color of the stick
c gstktext ...... POV-Ray texture index
c
c GMCONE ........ max. number of cones
c gncone ........ actual number of cones
c gcnpt ......... index to the end points of this cone (first point is
c                 center of the base and second point is the apex)
c gcnrad ........ radius of the cone base
c gcndefrgb ..... use default texture for STICKS?
c gcnrgbf ....... <r,g,b,f> color of the cone
c gcntext ....... POV-Ray texture index
c
c GMPOL ......... max. number of polygons
c GMSIDE ........ max. number of vertices in a polygon
c gnpol ......... actual number of polygons
c gnside ........ number of vertices of the polygon
c giside ........ index to the vertices of the polygon
c gnormal ....... (A,B,C,D) parameters of the Robert's equation of the
c                 plane. The sign of the parameters is established
c                 requiring that the normal points outside (i.e. the
c                 plane equation is positive for an inside point) in
c                 polyhedral faces.
c
c GMPLH ......... max. number of polyhedra
c GMFACE ........ max. number of faces in a polyhedron
c GMVERT ........ max. number of vertices in a polyhedron
c gnplh ......... actual number of polyhedra
c gnface ........ number of polyhedron faces
c giface ........ index to the polyhedron faces
c gctplh ........ index to the polyhedron center
c gnvert ........ number of vertices in the polyhedron
c givert ........ index to the polyhedron vertices
c gplhrgbf ...... <red,green,blue,filter> color specification for the
c                 polyhedron
c gplhtext ...... POV-Ray texture index
c
      integer           GMPT, GMBALL, GMSTK, GMCONE,
     &                  GMPOL, GMSIDE,
     &                  GMPLH, GMFACE, GMVERT
      parameter         (GMPT   = 20000)
      parameter         (GMBALL =  1000)
      parameter         (GMSTK  =  1000)
      parameter         (GMCONE = GMSTK)
      parameter         (GMPOL  =  2000)
      parameter         (GMSIDE =    10)
      parameter         (GMPLH  =  1000)
      parameter         (GMFACE =    20)
      parameter         (GMVERT =    20)
c
      integer           gnpt
      real*8            gxpt(GMPT), gypt(GMPT), gzpt(GMPT)
c
      integer           gnball, gballct(GMBALL), gballtext(GMBALL)
      real*8            gballrad(GMBALL), gballrgbf(4,GMBALL)
c
      integer           gnstk, gstkpt(2,GMSTK), gstktext(GMSTK)
      logical           gstkdefrgb(GMSTK)
      real*8            gstkrad(GMSTK), gstkrgbf(4,GMSTK)
c
      integer           gncone, gcnpt(2,GMCONE), gcntext(GMCONE)
      logical           gcndefrgb(GMCONE)
      real*8            gcnrad(GMCONE), gcnrgbf(4,GMCONE)
c
      integer           gnpol, gnside(GMPOL), giside(GMSIDE,GMPOL)
      real*8            gnormal(4,GMPOL)
c
      integer           gnplh, gnface(GMPLH), giface(GMFACE,GMPLH),
     &                  gnvert(GMPLH), givert(GMVERT,GMPLH),
     &                  gctplh(GMPLH), gplhtext(GMPLH)
      real*8            gplhrgbf(4,GMPLH)
c
      common /molgi/    gnpt, gnball, gballct, gballtext, gnstk, gstkpt,
     &                  gstktext, gncone, gcnpt, gcntext,
     &                  gnpol, gnside, giside, gnplh, gnface, giface,
     &                  gnvert, givert, gctplh, gplhtext
      common /molgl/    gstkdefrgb, gcndefrgb
      common /molgr/    gxpt, gypt, gzpt, gballrad, gballrgbf, gstkrad,
     &                  gstkrgbf, gcnrad, gcnrgbf, gnormal, gplhrgbf
c
c.....Image maps to be textured on the surface of balls:
c
c MFBUF ......... size of the buffer that will contain the name of all
c                 image map files.
c filebuffer .... name buffer.
c fbufp ......... pointer to the last character used in filebuffer.
c gballmaptype .. type of image map file (gif, tga, ...)
c gballfp ....... pointer to the first character of name of a file.
c
      integer           MFBUF
      parameter         (MFBUF = 1000)
      character*(MFBUF) filebuffer
      integer           fbufp
      integer           gballmaptype(GMBALL), gballfp(GMBALL)
      common /molti/    fbufp, gballmaptype, gballfp
      common /molts/    filebuffer
c
      integer           map_none, map_gif, map_tga
      parameter         (map_none = 0, map_gif = 1, map_tga = 2)
