c
c-----------------------------------------------------------------------
c
      subroutine molinit ()
c
c.....molinit - Declares initial values for some parameters used in the
c     molecular rendering modules.
c
      implicit none
      include           'datatm.inc'
      include           'molge.inc'
      include           'molec.inc'
      include           'spgcom.inc'
c
      integer           i
c
c.....Initialize the molecular graphics elements:
c
      gnpt = 0
      gnball = 0
      gnstk = 0
      gnpol = 0
      gnplh = 0
c
c.....Initialize the database of molecular structure:
c
      nneq = 0
      ncell = 0
      nhole = 0
      nneqclus = 0
      nclus = 0
      nmol = 0
c
      x0molclip = 0.0
      x1molclip = 1.0
      y0molclip = 0.0
      y1molclip = 1.0
      z0molclip = 0.0
      z1molclip = 1.0
c
      x0grfclip = 0.0
      x1grfclip = 1.0
      y0grfclip = 0.0
      y1grfclip = 1.0
      z0grfclip = 0.0
      z1grfclip = 1.0
c
c.....Initialize variables in the atomic properties data base:
c
      lengthtype = lbohr
      rconversion = 1d0
      grfactor = 0.3d0
      do i = -3, 7
         ionfactor(i) = 1.2d0
      enddo
      typrad = typION
c
c.....Initialize the image map variables:
c
      fbufp = 1
c
c.....Initialize the matrices that transform from centering to
c     primitive cells in the real and reciprocal space:
c
c     PRIMITIVE OR RHOMBOHEDRAL
c
      call fill (1, tocreal,  1.0d0,  0.0d0,  0.0d0,
     &                        0.0d0,  1.0d0,  0.0d0,
     &                        0.0d0,  0.0d0,  1.0d0)
      call fill (1, tocrec,   1.0d0,  0.0d0,  0.0d0,
     &                        0.0d0,  1.0d0,  0.0d0,
     &                        0.0d0,  0.0d0,  1.0d0)
      call fill (7, tocreal,  1.0d0,  0.0d0,  0.0d0,
     &                        0.0d0,  1.0d0,  0.0d0,
     &                        0.0d0,  0.0d0,  1.0d0)
      call fill (7, tocrec,   1.0d0,  0.0d0,  0.0d0,
     &                        0.0d0,  1.0d0,  0.0d0,
     &                        0.0d0,  0.0d0,  1.0d0)
c
c     F-CENTERED
c
      call fill (6, tocreal,  0.5d0,  0.5d0,  0.0d0,
     &                        0.0d0,  0.5d0,  0.5d0,
     &                        0.5d0,  0.0d0,  0.5d0)
      call fill (6, tocrec,   1.0d0, -1.0d0,  1.0d0,
     &                        1.0d0,  1.0d0, -1.0d0,
     &                       -1.0d0,  1.0d0,  1.0d0)
c
c     I-CENTERED
c
      call fill (5, tocreal,  0.5d0, -0.5d0,  0.5d0,
     &                        0.5d0,  0.5d0, -0.5d0,
     &                       -0.5d0,  0.5d0,  0.5d0)
      call fill (5, tocrec,   1.0d0,  1.0d0,  0.0d0,
     &                        0.0d0,  1.0d0,  1.0d0,
     &                        1.0d0,  0.0d0,  1.0d0)
c
c     A-CENTERED
c
      call fill (2, tocreal,  1.0d0,  0.0d0,  0.0d0,
     &                        0.0d0,  0.5d0,  0.5d0,
     &                        0.0d0, -0.5d0,  0.5d0)
      call fill (2, tocrec,   1.0d0,  0.0d0,  0.0d0,
     &                        0.0d0,  1.0d0, -1.0d0,
     &                        0.0d0,  1.0d0,  1.0d0)
c
c     B-CENTERED
c
      call fill (3, tocreal,  0.5d0,  0.0d0,  0.5d0,
     &                        0.0d0,  1.0d0,  0.0d0,
     &                       -0.5d0,  0.0d0,  0.5d0)
      call fill (3, tocrec,   1.0d0,  0.0d0, -1.0d0,
     &                        0.0d0,  1.0d0,  0.0d0,
     &                        1.0d0,  0.0d0,  1.0d0)
c
c     C-CENTERED
c
      call fill (4, tocreal,  0.5d0,  0.5d0,  0.0d0,
     &                       -0.5d0,  0.5d0,  0.0d0,
     &                        0.0d0,  0.0d0,  1.0d0)
      call fill (4, tocrec,   1.0d0, -1.0d0,  0.0d0,
     &                        1.0d0,  1.0d0,  0.0d0,
     &                        0.0d0,  0.0d0,  1.0d0)
c
      end
c
c-----------------------------------------------------------------------
c
      subroutine fill (i, xmat, x11, x12, x13,
     &                          x21, x22, x23,
     &                          x31, x32, x33)
c
c.....fill - fills the xmat 3x3 matrix with the elements given.
c
      real*8            xmat(7,3,3)
      real*8            x11, x12, x13, x21, x22, x23, x31, x32, x33
c
      xmat(i,1,1) = x11
      xmat(i,1,2) = x12
      xmat(i,1,3) = x13
      xmat(i,2,1) = x21
      xmat(i,2,2) = x22
      xmat(i,2,3) = x23
      xmat(i,3,1) = x31
      xmat(i,3,2) = x32
      xmat(i,3,3) = x33
c
      end
