c
c-----------------------------------------------------------------------
c
      subroutine molplbuild (
     &  xct, yct, zct,     !Polyhedron center
     &  useautodist,       !Invoke authomatic distance settings
     &  defdmin, defdmax,  !Limit distances otherwise
     &  nneig,             !Select type of atoms at the vertices
     &  namneig,           !Names of possible vertex neighbors
     &  usedefrgb,         !Use default colors?
     &  rgb,               !Use this color instead
     &  itexture,          !POV texture index
     &  dofaces,           !Plot the ployhedra as faces
     &  doedges,           !Plot the face edges
     &  stickrad,          !Edges radii
     &  uout)
c
c.....molplbuild - Create the coordination polyhedra around the
c     crystallographic position (xct,yct,zct). The vertices of the
c     polyhedron will be formed by all neighbors within a distance
c     >= dmin and <= dmax. The polyhedron is assumed to be convex
c     and simple (i.e. a convex hull).
c
      include           'implicit.inc'
      include           'consts.inc'
      include           'datatm.inc'
      include           'error.inc'
      include           'molec.inc'
      include           'molge.inc'
c
      character*(*)     namneig(*)
      integer           nneig, uout, itexture
      real*8            xct, yct, zct, defdmin, defdmax, rgb(4)
      real*8            stickrad
      logical           useautodist, usedefrgb
      logical           dofaces, doedges
c
c LOCAL VARIABLES:
c
      logical           equal, doit
c
      integer           mn
      parameter         (mn = 120)
      real*8            dmin, dmax
      real*8            rn(mn), xn(3,mn)
      integer           mpolig, msides
      parameter         (mpolig = 200, msides = 12)
      integer           n, npolig, nsides(mpolig), iside(msides,mpolig)
      real*8            hh, rpt(4), pmatrix(4)
      real*8            h1, h2, h3, x1, y1, z1, x2, y2, z2, x3, y3, z3
      real*8            xx1, yy1, zz1, xm, ym, zm, rr
      logical           inlist, doagain, reduce, discard, molpljoin
      logical           active(mpolig), con(mn,mn)
      integer           npt, i, j, j1, k, l, nstart
      integer           atmZi
      real*8            ri, qi
c
      dmin = defdmin
      dmax = defdmax
*mdc*if debug
*      write (0,*)
*      write (0,*) 'molplbuild: atom =', xct, yct, zct
*      write (0,*) 'Coord. radii =', rmin, radius
*      write (0,*) 'neig. =', nneig, (ineig(ii), ii = 1, nneig)
*mdc*endif
c
c.....Select all neighbors in the coordination polyhedron:
c
      n = 0
      do i = 1, nmol
c
c........First check distances of the candidate neighbors:
c
         xx1 = (xmol(i) - xct) * ared
         yy1 = (ymol(i) - yct) * bred
         zz1 = (zmol(i) - zct) * cred
         rr = sqrt (xx1*xx1 + yy1*yy1 + zz1*zz1 + 2d0 * 
     &        (xx1*yy1*cgamm + yy1*zz1*calph + zz1*xx1*cbeta))
         if (useautodist) then
            if (codemol(i).eq.codecrystal) then
               atmZi = zzneq(typmol(i))
               qi = qneq(typmol(i))
            else
               atmZi = zzclus(typmol(i))
               qi = qclus(typmol(i))
            endif
            if (typrad .eq. typION) then
               ri = (atmION(qi,atmZi)/bohr2pm) * ionfactor(qi)
            else if (typrad .eq. typvdW) then
               ri = (atmvdW(atmZi)/bohr2pm) * ionfactor(qi)
            else if (typrad .eq. typcov) then
               ri = (atmcov(atmZi)/bohr2pm) * ionfactor(qi)
            endif
            dmax = defdmax + ri
         endif
         doit = rr.le.dmax .and. rr.ge.dmin
c
c........Then check if the candidate is of the apropriate type:
c
         if (doit) then
            inlist = nneig.le.0
            k = 1
            do while (.not.inlist .and. k.le.nneig)
               if (codemol(i).eq.codecrystal) then
                  inlist = equal(namneig(k),namneq(typmol(i)))
               else
                  inlist = equal(namneig(k),namclus(typmol(i)))
               endif
               k = k + 1
            enddo
            doit = inlist
         endif
c
c........Add it to the neighbor list if all checks are satisfied:
c
         if (doit) then
            if (n.ge.mn) then
               call error ('molplbuild',
     &            'too many atoms in the buffer list. '//
     &            'Try to increase MN',faterr)
               return
            endif
            n = n + 1
            rn(n) = rr
            xn(1,n) = xmol(i)
            xn(2,n) = ymol(i)
            xn(3,n) = zmol(i)
*mdc*if debug
*            write (0,*) 'neig:', n, xn(1,n), xn(2,n), xn(3,n)
*mdc*endif
         endif
      enddo
c
c.....get the coordinates of the center of mass:
c
      xm = 0d0
      ym = 0d0
      zm = 0d0
      do i = 1, n
         xm = xm + xn(1,i)
         ym = ym + xn(2,i)
         zm = zm + xn(3,i)
      enddo
      xm = xm / n
      ym = ym / n
      zm = zm / n
*mdc*if debug
*      write (0,*) 'Center of mass:', xm, ym, zm
*mdc*endif
c
c.....Run over all non-null triangles between the selected points:
c
*mdc*if debug
*      write (0,*)
*      write (0,*) 'Start building polygons'
*mdc*endif
      npolig = 0
      do i = 1, n
         x1 = xn(1,i)
         y1 = xn(2,i)
         z1 = xn(3,i)
         do j = i+1, n
            x2 = xn(1,j)
            y2 = xn(2,j)
            z2 = xn(3,j)
            do k = j+1, n
               x3 = xn(1,k)
               y3 = xn(2,k)
               z3 = xn(3,k)
               h1 = (y3-y1)*(x2-x1) - (x3-x1)*(y2-y1)
               h2 = (z3-z1)*(x2-x1) - (x3-x1)*(z2-z1)
               h3 = (z3-z1)*(y2-y1) - (y3-y1)*(z2-z1)
c
c..............discard polygons between colinear points:
c
*mdc*if debug
*               if (abs(h1)+abs(h2)+abs(h3) .le. 1d-4) 
*     &            write (0,*) 'Polygon:', i, j, k, 'Colinear. Discard'
*mdc*endif
               if (abs(h1)+abs(h2)+abs(h3) .gt. 1d-4) then
c
c.................get the Robert's representation of this polygon
c                 (the sign is defined by requiring that the center of
c                 mass is *inside* the convex polyhedron):
c
                  pmatrix(1) = (y1-y2)*z3 + (y3-y1)*z2 + (y2-y3)*z1
                  pmatrix(2) = (x2-x1)*z3 + (x1-x3)*z2 + (x3-x2)*z1
                  pmatrix(3) = (x1-x2)*y3 + (x3-x1)*y2 + (x2-x3)*y1
                  pmatrix(4) = x1*(y2*z3-y3*z2) + y1*(x3*z2-x2*z3) +
     &                         z1*(x2*y3-x3*y2)
                  rpt(1) = xm
                  rpt(2) = ym
                  rpt(3) = zm
                  rpt(4) = 1d0
                  hh = 0d0
                  do l = 1, 4
                     hh = hh + rpt(l) * pmatrix(l)
                  enddo
                  if (hh.lt.0d0) then
                     do l = 1, 4
                        pmatrix(l) = -pmatrix(l)
                     enddo
                  endif
c
c.................if this triangle is an exterior face of the convex
c                 polyhedron, all the vertices must lie in the face
c                 or in the *positive* side of the plane:
c
                  discard = .false.
                  npt = 0
                  do while (.not.discard .and. npt.lt.n)
                     npt = npt + 1
                     rpt(1) = xn(1,npt)
                     rpt(2) = xn(2,npt)
                     rpt(3) = xn(3,npt)
                     rpt(4) = 1d0
                     hh = 0d0
                     do l = 1, 4
                        hh = hh + rpt(l) * pmatrix(l)
                     enddo
                     discard = hh.lt.-1d-5
                  enddo
c
c.................store out all non discarded triangles:
c
*mdc*if debug
*                  if (discard)
*     &            write (0,*) 'Polygon:', i, j, k, 'Not face. Discard'
*mdc*endif
                  if (.not.discard) then
                     npolig = npolig + 1
                     if (npolig.gt.mpolig) call error ('molplbuild',
     &                   'npolig > mpolig', faterr)
                     nsides(npolig) = 3
                     iside(1,npolig) = i
                     iside(2,npolig) = j
                     iside(3,npolig) = k
                     active(npolig) = .true.
*mdc*if debug
*                     write (0,*) 'Polyg: ', npolig, i, j, k
*mdc*endif
                  endif
               endif
            enddo
         enddo
      enddo
c
c.....Reduce coplanar polygons with two or more common vertices:
c
*mdc*if debug
*      write (0,*)
*      write (0,*) 'Reduction of polygons.'
*mdc*endif
      doagain = .true.
      do while (doagain)
c
c........repeat cycles until no more reductions occur:
c
         doagain = .false.
         do i = 1, npolig
            do j = i+1, npolig
               if (active(i) .and. active(j)) then
                  reduce = molpljoin (i, j, npolig, nsides, iside,
     &                     mpolig, msides, n, xn, mn)
                  if (reduce) then
                     doagain = .true.
                     active(j) = .false.
                  endif
               endif
            enddo
         enddo
*mdc*if debug
*         write (0,*)
*         write (0,*) 'New reduction cycle produces', npolig
*         do i = 1, npolig
*            if (active(i))
*     &      write (0,*) 'Polyg:', i, nsides(i),
*     &                  (iside(j,i),j=1,nsides(i))
*         enddo
*mdc*endif
      enddo
c
c.....Write out the vertices, polygons and polyhedra to the graphical
c     elements list:
c
      gnplh = gnplh + 1
      if (gnplh.gt.GMPLH) call error ('molplbuild',
     &                'Number of polyhedra larger than gmplh', faterr)
      gplhrgbf(1,gnplh) = rgb(1)
      gplhrgbf(2,gnplh) = rgb(2)
      gplhrgbf(3,gnplh) = rgb(3)
      gplhrgbf(4,gnplh) = rgb(4)
      gplhtext(gnplh) = itexture
c
c.....store the vertices:
c
      gnvert(gnplh) = n
      nstart = gnpt
      do i = 1, n
         gnpt = gnpt + 1
         if (gnpt.gt.GMPT) call error ('molplbuild',
     &                'Number of vertex larger than GMPT', faterr)
         gxpt(gnpt) = xn(1,i)
         gypt(gnpt) = xn(2,i)
         gzpt(gnpt) = xn(3,i)
         givert(i,gnplh) = gnpt
      enddo
c
c.....and the center of mass:
c
      gnpt = gnpt + 1
      if (gnpt.gt.GMPT) call error ('molplbuild',
     &             'Number of vertex larger than GMPT', faterr)
      gxpt(gnpt) = xm
      gypt(gnpt) = ym
      gzpt(gnpt) = zm
      gctplh(gnplh) = gnpt
c
c.....transform points to cartesian coordinates:
c
      do i = nstart + 1, gnpt
         gxpt(i) = gxpt(i)*crys2car(1) + gypt(i)*crys2car(2) +
     &             gzpt(i)*crys2car(3)
         gypt(i) = gypt(i)*crys2car(4) + gzpt(i)*crys2car(5)
         gzpt(i) = gzpt(i)*crys2car(6)
      enddo
c
c.....store the faces:
c
      gnface(gnplh) = 0
      do i = 1, npolig
         if (active(i)) then
            gnpol = gnpol + 1
            if (gnpol.gt.GMPOL) call error ('molplbuild',
     &                'Number of polygons larger than GMPOL', faterr)
            gnside(gnpol) = nsides(i)
            if (nsides(i).gt.GMSIDE) call error ('molplbuild',
     &        'Number of sides in a polygon larger than GMSIDE', faterr)
            do j = 1, nsides(i)
               giside(j,gnpol) = nstart + iside(j,i)
            enddo
            gnface(gnplh) = gnface(gnplh) + 1
            if (gnface(gnplh).gt.GMFACE) call error ('molplbuild',
     &            'Number of faces in a polyhedron larger than GMFACE',
     &            faterr)
            giface(gnface(gnplh),gnplh) = gnpol
         endif
      enddo
      if (.not. dofaces) gnplh = gnplh - 1
      if (doedges) then
         do i = 1, n
            do j = 1, n
               con(i,j) = .false.
            enddo
         enddo
         do i = 1, npolig
            if (active(i)) then
               do j = 1, nsides(i)
                  j1 = j + 1
                  if (j1 .gt. nsides(i)) j1 = 1
                  con(iside(j,i),iside(j1,i)) = .true.
                  con(iside(j1,i),iside(j,i)) = .true.
               enddo
            endif
         enddo
         do i = 1, n
            do j = 1, i-1
               if (con(i,j)) then
                  gnstk = gnstk + 1
                  if (gnstk.gt.GMSTK) call error ('molplbuild',
     &                'Number of sticks larger than GMSTK', faterr)
                  gstkpt(1,gnstk) = i + nstart
                  gstkpt(2,gnstk) = j + nstart
                  gstkrad(gnstk) = stickrad
                  gstkdefrgb(gnstk) = usedefrgb
                  gstkrgbf(1,gnstk) = rgb(1)
                  gstkrgbf(2,gnstk) = rgb(2)
                  gstkrgbf(3,gnstk) = rgb(3)
                  gstkrgbf(4,gnstk) = rgb(4)
                  gstktext(gnstk) = itexture
               endif
            enddo
         enddo
      endif
c
      end
