c
c-----------------------------------------------------------------------
c
      subroutine parametric (uin, uout)
c
c.....parametric - generation of a surface defined by means of a
c     parametric equation:
c
c             x(u,v), y(u,v), z(u,v)
c
c     where u and v are the two independent parameters.
c
      include           'implicit.inc'
      include           'error.inc'
      include           'strtools.inc'
      include           'molge.inc'
      include           'pov.inc'
      include           'tokens.inc'
c
      integer           uin, uout
c
c.....Polyhedra data structure:
c
      include           'polyhedron.inc'
c
      integer           i, j, i1, i2, j1, isup, jsup, upoints, vpoints
      integer           ii, nf, lp, lp1, lw, lc, ierr, ndeg
      real*8            u0, u1, ud, u, v0, v1, vd, v, xx
      real*8            sp, cp, st, ct, stsp, stcp
      logical           ok, done, uclosed, vclosed, depend
      logical           dopov, dooff, dovrml
      character*(600)   line
      character*(80)    word, title
      character*(80)    filepov, fileoff, filevrml
      character*(14)    uvarname, vvarname
c
      integer           leng
      logical           precompile, fgetvarname, expression, finputline
      logical           setdble, setint
      character*(80)    uppcase, getword
c
      character*(14)    coor_name(3), coor_type
      real*8            coor_value(3)
      integer           coor_mode
      integer           c_cartesian, c_spherical, c_cylindrical
      parameter         (c_cartesian=1, c_spherical=2, c_cylindrical=3)
c
      integer           dogrid
      integer           grid_none, grid_u, grid_v, grid_both
      parameter         (grid_none=0, grid_u=1, grid_v=2, grid_both=3)
      integer           gridtext, gnpt0, gnstk0
      logical           griddeftext
      real*8            gridrad, gridrgb(4), hsv(3)
c
      integer           vind
      vind(i,j) = (i-1)*vpoints + j
c
c.....Initialize calculator variables:
c
      call calcinit ()
      dogrid = grid_none
c
c.....Define default values:
c
      title = 'Plot of a parametric surface'
      coor_mode = c_cartesian
      coor_name(1) = 'X'//null
      coor_name(2) = 'Y'//null
      coor_name(3) = 'Z'//null
      uvarname = 'U'//null
      vvarname = 'V'//null
      u0 = -1
      u1 =  1
      upoints = 41
      uclosed = .false.
      v0 = -1
      v1 =  1
      vpoints = 41
      vclosed = .false.
      color_mode = color_none
      dopov = .false.
      filepov = 'tessel.pov'
      dooff = .false.
      fileoff = 'tessel.off'
      dovrml = .false.
      filevrml = 'tessel.wrl'
c
c.....Install default variables:
c
      xx = 0d0
      ii = 0
      call hinstall (coor_name(1), xx, ii, htvariable)
      call hinstall (coor_name(2), xx, ii, htvariable)
      call hinstall (coor_name(3), xx, ii, htvariable)
      call hinstall (uvarname, xx, ii, htvariable)
      call hinstall (vvarname, xx, ii, htvariable)
c
      write (uout,800)
c
c.....Read and interpret the next input file order:
c
      done = .false.
      do while (.not. done)
         ok = finputline (uin, line)
         if (.not. ok) call error ('parametric',
     &                 'Error looking for next input line', faterr)
         lp = 1
         word = getword (word, line, lp)
         word = uppcase (word)
         lw = leng(word)
c
c.......................................................................
         if (word(1:lw) .eq. 'MAP') then
c.......................................................................
c
c...........read in the type of coordinates used for the space:
c
            coor_type = getword (coor_type, line, lp)
            coor_type = uppcase (coor_type)
            lc = leng(coor_type)
            if (coor_type(1:lc) .eq. 'CARTESIAN') then
               coor_mode = c_cartesian
               coor_name(1) = 'X'//null
               coor_name(2) = 'Y'//null
               coor_name(3) = 'Z'//null
            else if (coor_type(1:lc) .eq. 'SPHERICAL') then
               coor_mode = c_spherical
               coor_name(1) = 'R'//null
               coor_name(2) = 'THETA'//null
               coor_name(3) = 'PHI'//null
            else if (coor_type(1:lc) .eq. 'CYLINDRICAL') then
               coor_mode = c_cylindrical
               coor_name(1) = 'R'//null
               coor_name(2) = 'PHI'//null
               coor_name(3) = 'Z'//null
            else
               call error ('parametric',
     &                     'space mapping type unknown', faterr)
            endif
            write (uout,805) coor_type(1:leng(coor_type))
            xx = 0d0
            ii = 0
            coor_name(1) = uppcase(coor_name(1))
            coor_name(2) = uppcase(coor_name(2))
            coor_name(3) = uppcase(coor_name(3))
            call hinstall (coor_name(1), xx, ii, htvariable)
            call hinstall (coor_name(2), xx, ii, htvariable)
            call hinstall (coor_name(3), xx, ii, htvariable)
c
c.......................................................................
         else if (word(1:lw) .eq. 'UNAME') then
c.......................................................................
c
c...........name for the u parametric coordinate:
c
            if (fgetvarname (uvarname, line, lp)) then
               uvarname = uppcase (uvarname)
               xx = 0d0
               ii = 0
               call hinstall (uvarname, xx, ii, htvariable)
            else
               call error ('parametric', 'Wrong name for u coord.',
     &                     faterr)
            endif
            write (uout,810) 'first (U)', uvarname(1:leng(uvarname))
c
c.......................................................................
         else if (word(1:lw) .eq. 'VNAME') then
c.......................................................................
c
c...........name for the v parametric coordinate:
c
            if (fgetvarname (vvarname, line, lp)) then
               vvarname = uppcase (vvarname)
               xx = 0d0
               ii = 0
               call hinstall (vvarname, xx, ii, htvariable)
            else
               call error ('parametric', 'Wrong name for v coord.',
     &                     faterr)
            endif
            write (uout,810) 'second (V)', vvarname(1:leng(vvarname))
c
c.......................................................................
         else if (word(1:lw) .eq. 'C1') then
c.......................................................................
c
c...........expression for the first space coordinate (i.e. x in
c           cartesians) in terms of the parametric variables:
c
            write (uout,815) coor_name(1)(1:leng(coor_name(1))),
     &                       line(lp:leng(line))
            if (.not. precompile (1, line, lp))
     &         call error ('parametric',
     &            'Error while precompiling coord. definition', faterr)
c
c.......................................................................
         else if (word(1:lw) .eq. 'C2') then
c.......................................................................
c
c...........expression for the second space coordinate (i.e. y in
c           cartesians) in terms of the parametric variables:
c
            write (uout,815) coor_name(2)(1:leng(coor_name(2))),
     &                       line(lp:leng(line))
            if (.not. precompile (2, line, lp))
     &         call error ('parametric',
     &            'Error while precompiling coord. definition', faterr)
c
c.......................................................................
         else if (word(1:lw) .eq. 'C3') then
c.......................................................................
c
c...........expression for the third space coordinate (i.e. z in
c           cartesians) in terms of the parametric variables:
c
            write (uout,815) coor_name(3)(1:leng(coor_name(3))),
     &                       line(lp:leng(line))
            if (.not. precompile (3, line, lp))
     &         call error ('parametric',
     &            'Error while precompiling coord. definition', faterr)
c
c.......................................................................
         else if (word(1:lw) .eq. 'URANGE') then
c.......................................................................
c
c...........Range for the U parameter:
c
            ok = setdble (u0, line, lp)
            if (.not. ok) call error ('parametric',
     &                                'urange line empty', warning)
            ok = ok .and. setdble (u1, line, lp)
            ok = ok .and. setint (upoints, line, lp)
            if (ok) then
               word = getword (word, line, lp)
               word = uppcase(word)
               if (word(1:leng(word)) .eq. 'CLOSED') then
                  uclosed = .true.
               else
                  uclosed = .false.
               endif
            endif
            write (uout,820) 'U', u0, u1, upoints, 'U', uclosed
c
c.......................................................................
         else if (word(1:lw) .eq. 'VRANGE') then
c.......................................................................
c
c...........Range for the V parameter:
c
            ok = setdble (v0, line, lp)
            if (.not. ok) call error ('parametric',
     &                                'vrange line empty', warning)
            ok = ok .and. setdble (v1, line, lp)
            ok = ok .and. setint (vpoints, line, lp)
            if (ok) then
               word = getword (word, line, lp)
               word = uppcase(word)
               if (word(1:leng(word)) .eq. 'CLOSED') then
                  vclosed = .true.
               else
                  vclosed = .false.
               endif
            endif
            write (uout,820) 'V', v0, v1, vpoints, 'V', vclosed
c
c.......................................................................
         else if (word(1:lw) .eq. 'COLORMODE') then
c.......................................................................
c
c...........Determine the color scheme to be used for the surface:
c
            write (uout,830) word(1:lw), line(lp:leng(line))
            word = getword (word, line, lp)
            word = uppcase (word)
            lw = leng (word)
            if (word(1:lw) .eq. 'NONE') then
               color_mode = color_none
            else if (word(1:lw) .eq. 'BINARY') then
               color_mode = color_binary
            else if (word(1:lw) .eq. 'GREY') then
               color_mode = color_grey
            else if (word(1:lw) .eq. 'FULL') then
               color_mode = color_full
            else if (word(1:lw) .eq. 'HUE') then
               color_mode = color_hue
            endif
            if (color_mode .eq. color_none) then
               ok = .true.
            else if (leng(line) .gt. lp) then
               ok = precompile (4, line, lp)
            else
               word = coor_name(3)
               lw = 1
               ok = precompile (4, word, lw)
            endif
            if (.not. ok) call error ('parametric',
     &            'Error while precompiling color function', faterr)
c
c.......................................................................
         else if (word(1:lw) .eq. 'GRID') then
c.......................................................................
c
c...........Add a grid on the parametric surface?
c
            write (uout,830) word(1:lw), line(lp:leng(line))
            gridrad = 0.05
            griddeftext = .true.
            gridrgb(1) = 1.0
            gridrgb(2) = 1.0
            gridrgb(3) = 0.0
            gridrgb(4) = 0.0
            gridtext = 0
            word = getword (word, line, lp)
            word = uppcase (word)
            lw = leng (word)
            ok = .true.
            do while (lw .gt. 0)
               if (word(1:lw) .eq. 'NONE') then
                  dogrid = grid_none
               else if (word(1:lw) .eq. 'U') then
                  dogrid = grid_u
               else if (word(1:lw) .eq. 'V') then
                  dogrid = grid_v
               else if (word(1:lw) .eq. 'BOTH') then
                  dogrid = grid_both
               else if (word(1:lw) .eq. 'RADIUS') then
                  ok = setdble (gridrad, line, lp)
               else if (word(1:lw) .eq. 'TEXTURE') then
                  ok = setint (gridtext, line, lp)
                  if (gridtext .gt. 0) then
                     if (.not. usepovtext(gridtext)) then
                        call error ('molecule',
     &                    'attempt to use a undeclared POV texture',
     &                    warning)
                        usepovtext(gridtext) = .true.
                     endif
                  endif
               else if (word(1:lw) .eq. 'RGB') then
                  ok = setdble (gridrgb(1), line, lp)
                  ok = ok .and. setdble (gridrgb(2), line, lp)
                  ok = ok .and. setdble (gridrgb(3), line, lp)
                  lp1 = lp
                  if (setdble (gridrgb(4), line, lp)) then
                     continue
                  else
                     lp = lp1
                     gridrgb(4) = 0.0
                  endif
                  if (ok) griddeftext = .false.
               else if (word(1:lw) .eq. 'HSV') then
                  ok = setdble (hsv(1), line, lp)
                  ok = ok .and. setdble (hsv(2), line, lp)
                  ok = ok .and. setdble (hsv(3), line, lp)
                  lp1 = lp
                  if (setdble (gridrgb(4), line, lp)) then
                     continue
                  else
                     lp = lp1
                     gridrgb(4) = 0.0
                  endif
                  if (ok) then
                     call hsv2rgb (hsv(1), hsv(2), hsv(3),
     &                             gridrgb(1), gridrgb(2), gridrgb(3))
                     griddeftext = .false.
                  endif
               else
                  ok = .false.
               endif
               word = getword (word, line, lp)
               word = uppcase (word)
               lw = leng (word)
            enddo
            if (.not. ok) then
               call error ('parametric',
     &            'Error on the GRID option. GRID ignored', warning)
               dogrid = grid_none
            endif
c
c.......................................................................
         else if (word(1:lw) .eq. 'POVRAY') then
c.......................................................................
c
c...........Select output of a povray file:
c
            dopov = .true.
            if (leng(line) .gt. lp) filepov = getword (word, line, lp)
c
c.......................................................................
         else if (word(1:lw) .eq. 'OFF') then
c.......................................................................
c
c...........Select output of an OFF file:
c
            dooff = .true.
            if (leng(line) .gt. lp) fileoff = getword (word, line, lp)
c
c.......................................................................
         else if (word(1:lw) .eq. 'VRML') then
c.......................................................................
c
c...........Select output of an VRML file:
c
            dovrml = .true.
            if (leng(line) .gt. lp) filevrml = getword (word, line, lp)
c
c.......................................................................
         else if (word(1:lw) .eq. 'SET') then
c.......................................................................
c
            call setvariables (line, lp, uout)
c
c.......................................................................
         else if (word(1:1) .eq. '#') then
c.......................................................................
c
            continue
c
c.......................................................................
         else if (word(1:lw) .eq. 'TITLE') then
c.......................................................................
c
            title = line(lp:leng(line))
c
c.......................................................................
         else if (word(1:lw) .eq. 'ENDPARAMETRIC') then
c.......................................................................
c
            done = .true.
c
c.......................................................................
         else
c.......................................................................
            call error ('parametric' , 'order not recognized '//
     &           line(1:leng(line)), faterr)
c
         endif
      enddo
c
      if (.not. (dooff .or. dopov .or. dovrml)) dooff = .true.
c
c.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.
c.+.+.+.+.+.+    DO THE REAL WORK                             +.+.+.+.+.
c.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.
c
c.....Determine all vertices:
c
      nvert = upoints * vpoints
      if (uclosed) then
         ud = (u1-u0) / upoints
      else
         ud = (u1-u0) / (upoints-1)
      endif
      if (vclosed) then
         vd = (v1-v0) / vpoints
      else
         vd = (v1-v0) / (vpoints-1)
      endif
*mdc*if debug
*      write (uout,100)
*mdc*endif
      do i = 1, upoints
         u = u0 + ud * (i-1)
         call hinstall (uvarname, u, 0, htvariable)
         do j = 1, vpoints
            v = v0 + vd * (j-1)
            call hinstall (vvarname, v, 0, htvariable)
            i1 = vind(i,j)
c
c...........evualuate the precompiled expressions to determine x,y,z:
c
            if (.not. expression (coor_value(1), 1, depend, ierr)) then
               call error ('parametric', 'error evaluating x', faterr)
            endif
            if (.not. expression (coor_value(2), 2, depend, ierr)) then
               call error ('parametric', 'error evaluating y', faterr)
            endif
            if (.not. expression (coor_value(3), 3, depend, ierr)) then
               call error ('parametric', 'error evaluating z', faterr)
            endif
            if (color_mode .ne. color_none) then
*mdc*if debug
*               write (uout,*) 'parametric(DBG) 1:', coor_value(1),
*     &                        coor_name(1)
*               write (uout,*) 'parametric(DBG) 1:', coor_value(2),
*     &                        coor_name(2)
*               write (uout,*) 'parametric(DBG) 1:', coor_value(3),
*     &                        coor_name(3)
*mdc*endif
               call hinstall (coor_name(1),coor_value(1),0,htvariable)
               call hinstall (coor_name(2),coor_value(2),0,htvariable)
               call hinstall (coor_name(3),coor_value(3),0,htvariable)
               if (.not. expression (fun_value(i1),4,depend,ierr)) then
                  call error ('parametric',
     &                        'error evaluating color function', faterr)
               endif
            endif
c
c...........pass the space coordinates to cartesian:
c
            if (coor_mode .eq. c_cartesian) then
               xvert(i1) = coor_value(1)
               yvert(i1) = coor_value(2)
               zvert(i1) = coor_value(3)
            else if (coor_mode .eq. c_spherical) then
               st = sin(coor_value(2))
               ct = cos(coor_value(2))
               stsp = st * sin(coor_value(3))
               stcp = st * cos(coor_value(3))
               if (coor_value(1).lt.0) coor_value(1) = -coor_value(1)
               xvert(i1) = coor_value(1) * stcp
               yvert(i1) = coor_value(1) * stsp
               zvert(i1) = coor_value(1) * ct
            else if (coor_mode .eq. c_cylindrical) then
               sp = sin(coor_value(2))
               cp = cos(coor_value(2))
               if (coor_value(1).lt.0) coor_value(1) = -coor_value(1)
               xvert(i1) = coor_value(1) * cp
               yvert(i1) = coor_value(1) * sp
               zvert(i1) = coor_value(3)
            endif
*mdc*if debug
*            write (uout,105) u,v,xvert(i1),yvert(i1),zvert(i1)
*mdc*endif
         enddo
      enddo
*mdc*if debug
* 100  format (1x,'--Evaluation of a parametric surface--'/
*     & 1x,'-----u---------v-----------x-------------y------',
*     &    '-------z------')
* 105  format (1x,2f10.4, 3f14.6)
*mdc*endif
c
c.....Store the faces:
c
      nvf = 4         !Each face will be a quadrilateral
      isw1 = 0
      nf = 0
      isup = upoints-1
      if (uclosed) isup = upoints
      jsup = vpoints-1
      if (vclosed) jsup = vpoints
      do i = 1, isup
         i1 = i + 1
         if (i1 .gt. upoints) i1 = 1
         do j = 1, jsup
            j1 = j + 1
            if (j1 .gt. vpoints) j1 = 1
            nf = nf + 1
            iface(1,nf,0) = vind(i,j)
            iface(2,nf,0) = vind(i1,j)
            iface(3,nf,0) = vind(i1,j1)
            iface(4,nf,0) = vind(i,j1)
         enddo
      enddo
c
      nface(0) = nf
c
c.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.
c.+.+.+.+.+.+    ADD THE OPTIONAL GRID                        +.+.+.+.+.
c.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.
c
      if (dogrid .ne. grid_none) then
         if (gnpt+upoints*vpoints .gt. GMPT) then
            call error ('parametric',
     &                  'GRID exceeds the max. number of mol. points',
     &                  warning)
            goto 10
         endif
         if (dogrid .eq. grid_u) then
            i = (upoints-1) * vpoints
         else if (dogrid .eq. grid_v) then
            i = (vpoints-1) * upoints
         else if (dogrid .eq. grid_both) then
            i = (upoints-1) * vpoints + (vpoints-1) * upoints
         endif
         if (gnstk+i .gt. GMSTK) then
            call error ('parametric',
     &                  'GRID exceeds the max. number of mol. sticks',
     &                  warning)
            goto 10
         endif
         gnpt0 = gnpt
         do i = 1, upoints
            do j = 1, vpoints
               i1 = vind(i,j)
               gnpt = gnpt + 1
               gxpt(gnpt) = xvert(i1)
               gypt(gnpt) = yvert(i1)
               gzpt(gnpt) = zvert(i1)
            enddo
         enddo
         gnstk0 = gnstk
         if (dogrid.eq.grid_u .or. dogrid.eq.grid_both) then
            do j = 1, vpoints
               do i = 1, upoints-1
                  i1 = gnpt0 + vind(i,j)
                  i2 = gnpt0 + vind(i+1,j)
                  gnstk = gnstk + 1
                  gstkpt(1,gnstk) = i1
                  gstkpt(2,gnstk) = i2
                  gstkrad(gnstk) = gridrad
                  gstkdefrgb(gnstk) = griddeftext
                  gstkrgbf(1,gnstk) = gridrgb(1)
                  gstkrgbf(2,gnstk) = gridrgb(2)
                  gstkrgbf(3,gnstk) = gridrgb(3)
                  gstkrgbf(4,gnstk) = gridrgb(4)
                  gstktext(gnstk) = gridtext
               enddo
            enddo
         endif
         if (dogrid.eq.grid_v .or. dogrid.eq.grid_both) then
            do i = 1, upoints
               do j = 1, vpoints-1
                  i1 = gnpt0 + vind(i,j)
                  i2 = gnpt0 + vind(i,j+1)
                  gnstk = gnstk + 1
                  gstkpt(1,gnstk) = i1
                  gstkpt(2,gnstk) = i2
                  gstkrad(gnstk) = gridrad
                  gstkdefrgb(gnstk) = griddeftext
                  gstkrgbf(1,gnstk) = gridrgb(1)
                  gstkrgbf(2,gnstk) = gridrgb(2)
                  gstkrgbf(3,gnstk) = gridrgb(3)
                  gstkrgbf(4,gnstk) = gridrgb(4)
                  gstktext(gnstk) = gridtext
               enddo
            enddo
         endif
         write (uout,840) gnpt-gnpt0, gnstk-gnstk0
      endif
c
c.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.
c.+.+.+.+.+.+    OUTPUT THE RESULTS                           +.+.+.+.+.
c.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.+.
c
 10   continue
      if (dovrml) then
         call writevrml1 (filevrml, title, uout)
      endif
      if (dooff) then
         call writeoff (fileoff, title, uout)
      endif
      if (dopov) then
         call getnormals (ndeg, uout)
         if (ndeg .gt. 0) then
            call removedeg (ndeg, uout)
            call getnormals (ndeg, uout)
         endif
         call writepov (filepov, title, uout)
      endif
c
c.....FORMATS:
c
 800  format (/
     & 1x, 'PARAMETRIC: Creation of an arbitrary surface defined'/
     & 1x, '            parametrically as {x(u,v),y(u,v),z(u,v)}.'/)
 805  format (
     & 1x, 'The space will be described in ', a, ' coordinates')
 810  format (
     & 1x, 'Name for the ', a, ' parameter: ', a)
 815  format (
     & 1x, 'Expression for ', a, ': ', a)
 820  format (
     & 1x, 'Range for the ', a, ' parameter (first,last,points): ',
     &     1p, 2e13.6, 0p, i4/
     & 1x, 'Is the grid for ', a, ' closed? ', l3)
 830  format (1x, a, a)
 840  format (
     & 1x, 'Grid added with ', i4, ' points and ', i4, ' sticks')
c
      end
