c
c-----------------------------------------------------------------------
c
      logical function polyjoin (
     &    ipol, jpol, !candidates to be joined
     &    npolig,     !number of polygons
     &    nsides,     !number of sides in each polygon
     &    iside,      !index to the first vertex on each side
     &    mpolig,     !max. number of polygons
     &    msides,     !max. number of sides
     &    nn,         !
     &    xn,         !coordinates of the vertices
     &    mn)         !max. number of vertices
c
c.....polyjoin - glues together the polygons ipol and jpol when coplanar
c     and they have two or more vertices in common.
c
c GLOBAL DATA:
c
      include           'implicit.inc'
      include           'molec.inc'
      include           'error.inc'
c
c PARAMETERS:
c
      integer           mpolig, msides, mn
      integer           ipol, jpol, npolig, nsides(mpolig),
     &                  iside(msides,mpolig), nn
      real*8            xn(3,mn)
c
c LOCAL VARIABLES:
c
      real*8            EPS
      parameter         (EPS = 1d-6)
      integer           mpts
      parameter         (mpts = 20)
      integer           mpair
      parameter         (mpair = (mpts*(mpts-1))/2)
c
      real*8            d1, dx, dy, dz, x1, x2, x3, y1, y2, y3,
     &                  z1, z2, z3, rr
      real*8            xnor2(3,0:mpair)
      real*8            finside, fk, xct, yct
      integer           i, j, i1, ncom, npts, ipts(mpts)
      integer           dir1, dir2, k, n
      logical           iused(mpts), newpoint
      logical           connect(mpts,mpts), con, done, reject
c
c.....Get the equation of the ipol polygon: d1 + dx*x + dy*y + dz*z = 0
c
      x1 = xn(1,iside(1,ipol))
      y1 = xn(2,iside(1,ipol))
      z1 = xn(3,iside(1,ipol))
      x2 = xn(1,iside(2,ipol))
      y2 = xn(2,iside(2,ipol))
      z2 = xn(3,iside(2,ipol))
      x3 = xn(1,iside(3,ipol))
      y3 = xn(2,iside(3,ipol))
      z3 = xn(3,iside(3,ipol))
      d1 = x1*(y2*z3-y3*z2) + y1*(x3*z2-x2*z3) + z1*(x2*y3-x3*y2)
      dx = (y1-y2)*z3 + (y3-y1)*z2 + (y2-y3)*z1
      dy = (x2-x1)*z3 + (x1-x3)*z2 + (x3-x2)*z1
      dz = (x1-x2)*y3 + (x3-x1)*y2 + (x2-x3)*y1
c
c.....Check if the ipol and jpol polygons are coplanar:
c
      do i = 1, nsides(jpol)
         i1 = iside(i,jpol)
         rr = dx*xn(1,i1) + dy*xn(2,i1) + dz*xn(3,i1) + d1
         if (abs(rr) .gt. EPS) then
c
c...........a single point out of the plane precludes coplanarity:
c
            polyjoin = .false.
            return
         endif
      enddo
c
c.....Check for common points in both polygons:
c     (It is assumed that a single polygon has no repeated points)
c
      ncom = 0
      npts = 0
      do i = 1, nsides(ipol)
         npts = npts + 1
         ipts(npts) = iside(i,ipol)
         iused(npts) = .false.
      enddo
      do i = 1, nsides(jpol)
         newpoint = .true.
         j = 1
         do while (newpoint .and. j.le.npts)
            newpoint = iside(i,jpol) .ne. ipts(j)
            j = j + 1
         enddo
         if (newpoint) then
            npts = npts + 1
            ipts(npts) = iside(i,jpol)
            iused(npts) = .false.
         else
            ncom = ncom + 1
         endif
      enddo
c
c.....two or more common points are required to join the polygons:
c
      if (ncom .lt. 2) then
         polyjoin = .false.
         return
      endif
************************************************************************
************************************************************************
c
c.....Connect the points to form a convex polygon. This is a version of
c     the convex hull in 2 dimensions. We will use a very simple and
c     quite inefficient method ... but the number of points is *very*
c     small.
c
c.....The 3D points will be projected on a cartesian plane. Choose a
c     plane such that no two points coincide:
c
      xnor2(1,0) = 0d0
      xnor2(2,0) = 0d0
      xnor2(3,0) = 0d0
      n = 0
      do i = 1, npts
         do j = 1, i-1
            n = n + 1
            xnor2(1,n) = abs(xn(1,ipts(i)) - xn(1,ipts(j)))
            xnor2(2,n) = abs(xn(2,ipts(i)) - xn(2,ipts(j)))
            xnor2(3,n) = abs(xn(3,ipts(i)) - xn(3,ipts(j)))
            xnor2(1,0) = xnor2(1,0) + xnor2(1,n)
            xnor2(2,0) = xnor2(2,0) + xnor2(2,n)
            xnor2(3,0) = xnor2(3,0) + xnor2(3,n)
         enddo
      enddo
      dir1 = 1
      dir2 = 2
      i = 1
      done = .false.
      do while (.not.done)
         reject = xnor2(dir1,0).le.EPS .or. xnor2(dir2,0).le.EPS
         j = 1
         do while (.not.reject .and. j.le.n)
            reject = xnor2(dir1,j)+xnor2(dir2,j) .lt. EPS
            j = j + 1
         enddo
         if (reject) then
            dir1 = dir1 + 1
            dir2 = mod(dir2,3) + 1
            i = i + 1
            if (i.gt.3) then
*mdc*if debug
*               write (6,*) 'DBG(polyjoin): Points to connect', npts
*               do k = 1, npts
*                  write (6,*) ipts(k), (xn(n,ipts(k)), n = 1, 3)
*               enddo
*mdc*endif
               call error ('polyjoin', 'Algorithm error 1!', faterr)
            endif
         else
            done = .true.
         endif
      enddo
c
c.....The next point SHOULD be INSIDE the convex polygon:
c
      xct = xn(dir1,ipts(1))
      yct = xn(dir2,ipts(1))
      do i = 2, npts
         xct = xct + xn(dir1,ipts(i))
         yct = yct + xn(dir2,ipts(i))
      enddo
      xct = xct / npts
      yct = yct / npts
c
c.....Determine the points connected by a perimetral segment
c     of the convex polygon. To do that, check every pair of points.
c     If a segment is in the perimeter of the polygon, all the points
c     have to be on the same side of the plane:
c
      do i = 1, npts
         connect(i,i) = .false.
         x1 = xn(dir1,ipts(i))
         y1 = xn(dir2,ipts(i))
         do j = 1, i-1
            x2 = xn(dir1,ipts(j))
            y2 = xn(dir2,ipts(j))
            dx = y1 - y2
            dy = x2 - x1
            d1 = (x1*y2-x2*y1)
c
c...........the position of the (xct,yct) point is INSIDE:
c
            finside = dx*xct + dy*yct + d1
            if (abs(finside) .ge. EPS) then
               con = .true.
            else
               con = .false.
            endif
            k = 1
            do while (con .and. k.le.npts)
c
c..............a single point outside means that i-j are not connected:
c
               if (k.eq.i .or. k.eq.j) then
                  k = k + 1
               else
                  fk = dx*xn(dir1,ipts(k)) + dy*xn(dir2,ipts(k)) + d1
                  if (fk*finside .lt. -EPS) then
                     con = .false.
                  else
                     k = k + 1
                  endif
               endif
            enddo
            if (con) then
               connect(i,j) = .true.
               connect(j,i) = .true.
            else
               connect(i,j) = .false.
               connect(j,i) = .false.
            endif
         enddo
      enddo
c
c.....Get a point on the perimeter:
c
      i = 1
      done = .false.
      do while (.not.done)
         con = .false.
         j = 1
         do while (.not.con .and. j.le.npts)
            con = connect(i,j)
            j = j + 1
         enddo
         if (con) then
            done = .true.
         else
            i = i + 1
         endif
         if (i.gt.npts) then
*mdc*if debug
*            write (6,*) 'DBG(polyjoin): npts', npts
*            do k = 1, npts
*               write (6,*) ipts(k), (connect(k,n), n = 1, npts)
*            enddo
*            write (6,*) 'dirs: ', dir1, dir2
*            write (6,*) 'xct,yct ', xct, yct
*            write (6,*) 'points:'
*            do k = 1, npts
*               write (6,*) ipts(k), (xn(n,ipts(k)), n = 1, 3)
*            enddo
*mdc*endif
            call error ('polyjoin', 'Algorithm error 2!', faterr)
         endif
      enddo
c
c.....Get all points in the perimeter, starting on the point *i*:
c
      n = 1
      iside(n,ipol) = ipts(i)
      iused(i) = .true.
      done = .false.
      do while (.not.done)
         j = 1
         con = .false.
         do while (.not.con .and. j.le.npts)
            if (connect(i,j) .and. .not.iused(j)) then
               con = .true.
            else
               j = j + 1
            endif
         enddo
         if (con) then
            n = n + 1
            iside(n,ipol) = ipts(j)
            iused(j) = .true.
            i = j
         else
            done = .true.
         endif
      enddo
      nsides(ipol) = n
      nsides(jpol) = 0
      polyjoin = .true.
      return
      end
