c
c-----------------------------------------------------------------------
c
      subroutine readbasin (line, lp, uout)
c
c.....readbasin - read the polyhedron description contained in one
c     or several files, and prepares OFF or POV-Ray data files.
c
      include           'implicit.inc'
      include           'error.inc'
      include           'polyhedron.inc'
      include           'strtools.inc'
c
      character*(*)     line
      integer           lp, uout
c
      integer           luo, ind, ndeg, lw, i, j, k
      integer           mpropty, mfile
      parameter         (mpropty = 50, mfile = 50)
      real*8            prop(mpropty)
      integer           npropty, nfile, nprop
      integer           nedge
      logical           done, ok
      character*(80)    word, word1, word2, filename(mfile), line1
c
      integer           out_mode, out_OFF, out_POV, out_VRML
      parameter         (out_OFF=100, out_POV=101, out_VRML=102)
c
      integer           cook_linear, cook_atan, cook_tanh, cook_log
      parameter         (cook_linear = 1, cook_atan = 2, cook_tanh = 3,
     &                  cook_log = 4)
      real*8            cookfactor
      integer           cook_method
      character*(6)     cook_name
c
      logical           doclip
      real*8            clipmin, clipmax
c
      integer           leng, fopen
      logical           commentline, setint, setdble
      character*(80)    getword, uppcase
c
c.....Defaults:
c
      nprop = 0
      out_mode = out_OFF
      nfile = 0
      color_mode = color_none
      cook_method = cook_linear
      doclip = .false.
c
c.......................................................................
c.....Recover the order parameters:
c.......................................................................
c
      word = getword (word, line, lp)
      lw = leng(word)
      done = lw .le. 0
      do while (.not. done)
         word = uppcase (word)
c
         if (word(1:lw) .eq. 'PROPERTY') then
c
            ok = setint (nprop, line, lp)
c
         else if (word(1:lw) .eq. 'WRITE') then
c
            ok = .true.
            word = getword (word, line, lp)
            word = uppcase (word)
            lw = leng(word)
            if (word(1:lw) .eq. 'OFF') then
               out_mode = out_OFF
            else if (word(1:lw) .eq. 'POVRAY' .or.
     &               word(1:lw) .eq. 'POV'         ) then
               out_mode = out_POV
            else if (word(1:lw) .eq. 'VRML') then
               out_mode = out_VRML
            else
               ok = .false.
            endif
c
         else if (word(1:lw) .eq. 'COLORMODE') then
c
            ok = .true.
            word = getword (word, line, lp)
            word = uppcase (word)
            lw = leng(word)
            if (word(1:lw) .eq. 'NONE') then
               color_mode = color_none
            else if (word(1:lw) .eq. 'BINARY') then
               color_mode = color_binary
            else if (word(1:lw) .eq. 'FULL') then
               color_mode = color_full
            else if (word(1:lw) .eq. 'GREY') then
               color_mode = color_grey
            else
               ok = .false.
            endif
c
         else if (word(1:lw) .eq. 'COLORSCALE') then
c
            ok = .true.
            word = getword (word, line, lp)
            word = uppcase (word)
            lw = leng(word)
            if (word(1:lw) .eq. 'LINEAR') then
               cookfactor = 1d0
               cook_method = cook_linear
            else if (word(1:lw) .eq. 'LOG') then
               cookfactor = 1d0
               cook_method = cook_log
            else if (word(1:lw) .eq. 'ATAN') then
               cookfactor = 1d0
               cook_method = cook_atan
            else if (word(1:lw) .eq. 'TANH') then
               cookfactor = 1d0
               cook_method = cook_tanh
            else if (word(1:lw) .eq. 'ATANF') then
               cook_method = cook_atan
               if (setdble(cookfactor, line, lp)) then
                  ok = .true.
               else
                  cookfactor = 1d0
               endif
            else if (word(1:lw) .eq. 'TANHF') then
               cook_method = cook_tanh
               if (setdble(cookfactor, line, lp)) then
                  ok = .true.
               else
                  cookfactor = 1d0
               endif
            else
               ok = .false.
            endif
c
         else if (word(1:lw) .eq. 'CLIP') then
c
            ok = setdble (clipmin, line, lp)
            ok = ok .and. setdble (clipmax, line, lp)
            if (ok) doclip = .true.
c
         else if (word(1:lw) .eq. 'FILES') then
c
            nfile = 0
            word = getword (word, line, lp)
            lw = leng(word)
            do while (lw .gt. 0)
               nfile = nfile + 1
               filename(nfile) = word
               word = getword (word, line, lp)
               lw = leng(word)
            enddo
            ok = nfile .gt. 0
c
         endif
c
         if (.not. ok) then
           call error ('readbasin', 'wrong sintaxis in BASINPLOT',
     &          faterr)
           return
         endif
c
         word = getword (word, line, lp)
         lw = leng(word)
         done = lw .le. 0
c
      enddo
c
      if (nfile .le. 0) ok = .false.
      if (.not. ok) then
        call error ('readbasin', 'wrong sintaxis in BASINPLOT',
     &    faterr)
        return
      endif
      if (nprop .le. 0) color_mode = color_none
c
c-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
c
c.....When dealing with several files together, determine the absolute
c     minimun and maximum value of the property and use it to scale
c     the surface colors:
c
      if ((color_mode.eq.color_full .or. color_mode.eq.color_grey) .and.
     &    nfile.gt.1) then
         useminmax = .true.
         do i = 1, nfile
c
c...........Open the input file:
c
            if (fopen (luo, filename(i), ioread) .eq. ioerror) then
               write (uout,*) 'readbasin: Data file is ',
     &                        filename(i)(1:leng(filename(i)))
               call error ('readbasin',
     &                     'I/O error when opening points file',
     &                     faterr)
               return
            endif
c
            isw1 = 0
c
c...........Skip initial comment lines, if any:
c
            do while (commentline(luo, line))
               continue
            enddo
c
c...........Start reading:
c
            read (luo,*) nvert, nface(isw1), nedge
            read (luo,*) npropty
            read (luo,'(a)') line1
c
            do j = 1, nvert
c
c..............Get the vertex coordinates and the properties:
c
               read (luo,*) xvert(j), yvert(j), zvert(j),
     &                      (prop(k), k = 1, npropty)
               fun_value(j) = prop(nprop)
            enddo
c
c...........Determine min and max function values:
c
            if (i .eq. 1) then
               fun_min = fun_value(1)
               fun_max = fun_value(1)
            endif
            do j = 1, nvert
               if (fun_min .gt. fun_value(j)) fun_min = fun_value(j)
               if (fun_max .lt. fun_value(j)) fun_max = fun_value(j)
            enddo
c
            call fclose (luo)
c
         enddo
      endif
c
c.....clip the min and max values of the function if neccesary:
c
      if (doclip) then
         fun_min = max (fun_min, clipmin)
         fun_max = min (fun_max, clipmax)
      endif
c
c.....and apply a function scaling if the user ordered so:
c
      if (cook_method .eq. cook_log) then
         if (fun_min .le. 0d0) then
            call error ('readbasin',
     &           'LOG scale cannot be used on negative values', warning)
            cook_method = cook_linear
            cook_name = 'LINEAR'
         else
            fun_min = log(fun_min)
            fun_max = log(fun_max)
            cook_name = 'LOG'
         endif
      else if (cook_method .eq. cook_atan) then
         fun_min = atan(cookfactor * fun_min)
         fun_max = atan(cookfactor * fun_max)
         cook_name = 'ATAN'
      else if (cook_method .eq. cook_tanh) then
         fun_min = tanh(cookfactor * fun_min)
         fun_max = tanh(cookfactor * fun_max)
         cook_name = 'TANH'
      else
         cook_name = 'LINEAR'
      endif
c
c-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
c
c.....Read again all data files to produce the final output:
c
      do i = 1, nfile
c
c........Open the input file:
c
         if (fopen (luo, filename(i), ioread) .eq. ioerror) then
            write (uout,*) 'readbasin: Data file is ',
     &                     filename(i)(1:leng(filename(i)))
            call error ('readbasin',
     &                  'I/O error when opening points file',
     &                  faterr)
            return
         endif
c
         isw1 = 0
c
c........Skip initial comment lines, if any:
c
         do while (commentline(luo, line))
            continue
         enddo
c
c........Start reading:
c
         read (luo,*) nvert, nface(isw1), nedge
         read (luo,*) npropty
         read (luo,'(a)') line1
c
         do j = 1, nvert
c
c...........Get the vertex coordinates and the properties:
c
            read (luo,*) xvert(j), yvert(j), zvert(j),
     &                   (prop(k), k = 1, npropty)
            if (nprop.gt.0) fun_value(j) = prop(nprop)
         enddo
c
c........clip and scale the property if appropriate:
c
         if (nprop.gt.0) then
            if (doclip) then
               do j = 1, nvert
                  if (fun_value(j) .lt. clipmin) fun_value(j) = clipmin
                  if (fun_value(j) .gt. clipmax) fun_value(j) = clipmax
               enddo
            endif
            if (cook_method .eq. cook_log) then
               do j = 1, nvert
                  fun_value(j) = log(fun_value(j))
               enddo
            else if (cook_method .eq. cook_atan) then
               do j = 1, nvert
                  fun_value(j) = atan(cookfactor * fun_value(j))
               enddo
            else if (cook_method .eq. cook_tanh) then
               do j = 1, nvert
                  fun_value(j) = tanh(cookfactor * fun_value(j))
               enddo
            endif
         endif
c
c........Read in the faces:
c
         do j = 1, nface(isw1)
            read (luo,*) nvf, (iface(k,j,isw1), k = 1, nvf)
         enddo
         do k = 1, nvf
            do j = 1, nface(isw1)
               iface(k,j,isw1) = iface(k,j,isw1) + 1
            enddo
         enddo
c
         call fclose (luo)
c
c........Determine the name of the property that will be plotted:
c
         k = 1
         do j = 1, nprop
            word = getword (word, line1, k)
         enddo
c
         write (uout,100) filename(i)(1:leng(filename(i))), nvert,
     &         nface(isw1), fun_min, fun_max, word(1:leng(word)),
     &         doclip, clipmin, clipmax, cook_name, cookfactor
c
c........Call to the routines that produce the final output:
c
         word1 = 'Surface color determined by the property: '//
     &           word(1:leng(word))
         ind = index(filename(i), '.basin') - 1
         if (ind .le. 0) ind = leng(filename(i))
c
         if (out_mode .eq. out_OFF) then
            if (color_mode .eq. color_none) then
               word2 = filename(i)(1:ind)//'.off'
            else
               word2 = filename(i)(1:ind)//'.coff'
            endif
            call writeoff (word2, word1, uout)
         else if (out_mode .eq. out_POV) then
            call getnormals (ndeg, uout)
            if (ndeg .gt. 0) then
               call removedeg (ndeg, uout)
               call getnormals (ndeg, uout)
            endif
            word2 = filename(i)(1:ind)//'.pov'
            call writepov (word2, word1, uout)
         else if (out_mode .eq. out_VRML) then
            word2 = filename(i)(1:ind)//'.wrl'
            call writevrml1 (word2, word1, uout)
         endif
c
      enddo
c
c-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-
c
      useminmax = .false.
c
 100  format (/
     & 1x, 'READBASIN: Reading start points from a file'/
     & 1x, 'Data file name: ', a/
     & 1x, 'Number of vertices read in: ', i4/
     & 1x, 'Number of faces read in:    ', i4/
     & 1x, 'Min and max function value: ', 1p, 2e15.8, 0p/
     & 1x, 'Surface color according to the property: ', a/
     & 1x, 'Clip function values?       ', l4/
     & 1x, 'Clip range (min..max):      ', 1p, 2e15.8, 0p/
     & 1x, 'Colorscale used:   ', a/
     & 1x, 'Colorscale factor: ', f12.4 )
c
      end
