c
c-----------------------------------------------------------------------
c
      subroutine removedeg (ndegenerate, uout)
c
c.....removedeg - determine normal vectors of all the faces and
c     average normals for each vertex.
c
      include           'implicit.inc'
c
c.....Polyhedra data structure:
c
      include           'polyhedron.inc'
      include           'error.inc'
      include           'normals.inc'
c
      integer           ndegenerate
      integer           uout
c
      real*8            EPSILON
      parameter         (EPSILON = 1d-20)
c
      integer           i, i1, i2, i3, nn, newface
      real*8            x12, y12, z12, x13, y13, z13, aa, bb, cc, xn
c
c.....The polyhedron is assumed to be made of clockwise triangles.
c     If not, it will be reconstructed as a sum of triangles:
c
      if (nvf .eq. 3) then
         continue
      else if (nvf .eq. 4) then
         nn = nface(isw1)
         do i = 1, nface(isw1)
            nn = nn + 1
            iface(1,nn,isw1) = iface(3,i,isw1)
            iface(2,nn,isw1) = iface(4,i,isw1)
            iface(3,nn,isw1) = iface(1,i,isw1)
         enddo
         nface(isw1) = nn
         nvf = 3
      else
         call error ('removedeg', 'faces can only be traingles or'//
     &               ' quadrilaterals', faterr)
      endif
c
c.....Determine the normal vector for each triangular face:
c
      newface = 0
      ndegenerate = 0
      do i = 1, nface(isw1)
         i1 = iface(1,i,isw1)
         i2 = iface(2,i,isw1)
         i3 = iface(3,i,isw1)
         x12 = xvert(i1) - xvert(i2)
         y12 = yvert(i1) - yvert(i2)
         z12 = zvert(i1) - zvert(i2)
         x13 = xvert(i1) - xvert(i3)
         y13 = yvert(i1) - yvert(i3)
         z13 = zvert(i1) - zvert(i3)
         aa = (y12*z13 - z12*y13)
         bb = (z12*x13 - x12*z13)
         cc = (x12*y13 - y12*x13)
         xn = aa*aa + bb*bb + cc*cc
         if (xn .ge. EPSILON) then
            newface = newface + 1
            iface(1,newface,isw1) = iface(1,i,isw1)
            iface(2,newface,isw1) = iface(2,i,isw1)
            iface(3,newface,isw1) = iface(3,i,isw1)
         else
            ndegenerate = ndegenerate + 1
         endif
      enddo
      nface(isw1) = newface
c
      write (uout,100) ndegenerate, newface
 100  format (/
     & 1x, 'REMOVEDEG: Remove degenerate faces.'/
     & 1x, 'Number of degenerate faces found: ', i6/
     / 1x, 'Final number of triangular faces: ', i6 )
c
      end
