c
c-----------------------------------------------------------------------
c
      subroutine sphere (x0, y0, z0,   !center of the sphere
     &                   radius,       !radius
     &                   ipol,         !polyhedra: 1=icosahedra ...
     &                   x, y, z,      !vertex coordinates
     &                   nv,           !number of vertex
     &                   nf,           !number of faces
     &                   nvf,          !actual # of vertex per face
     &                   mvf,          !max. # of vertex per face
     &                   ifa)          !index of vertices on each face
c
c.....sphere - returns the coordinates of a *regular* polyhedron
c     that approximates a sphere centered at the point (x0,y0,z0)
c     of the given radius.
c
c.....Available polyhedra:
c         1 .......... icosahedron (vert=12, face=20)
c         2 .......... cube        (vert= 8, face= 6)
c         3 .......... octahedron  (vert= 6, face= 8)
c         4 .......... tetrahedron (vert= 4, face= 4)
c
      implicit none
      include           'consts.inc'
      real*8            x0, y0, z0, x(*), y(*), z(*), radius
      integer           ipol, nv, nf, nvf, mvf, ifa(mvf,*)
c
      real*8            tau, phi, rad0
      integer           i
c
c.....Golden ratio:
c
      tau = (1d0 + sqrt(5d0)) / 2d0
c
      if (ipol .eq. 1) then
c
c........icosahedron:
c
         phi = tau - 1d0
         rad0 = sqrt(3d0 - tau)
         nv = 12
         x( 1) = +1   
         y( 1) =  0   
         z( 1) = +phi
         x( 2) = +1   
         y( 2) =  0   
         z( 2) = -phi
         x( 3) = -1   
         y( 3) =  0   
         z( 3) = +phi
         x( 4) = -1   
         y( 4) =  0   
         z( 4) = -phi
         x( 5) =  0   
         y( 5) = +phi 
         z( 5) = +1
         x( 6) =  0   
         y( 6) = -phi 
         z( 6) = +1
         x( 7) =  0   
         y( 7) = +phi 
         z( 7) = -1
         x( 8) =  0   
         y( 8) = -phi 
         z( 8) = -1
         x( 9) = +phi 
         y( 9) = +1   
         z( 9) =  0
         x(10) = -phi 
         y(10) = +1   
         z(10) =  0
         x(11) = +phi 
         y(11) = -1   
         z(11) =  0
         x(12) = -phi 
         y(12) = -1   
         z(12) =  0
         nvf = 3
         nf = 20
         ifa(1, 1) =  5 
         ifa(2, 1) =  9 
         ifa(3, 1) =  1
         ifa(1, 2) = 10 
         ifa(2, 2) =  7 
         ifa(3, 2) =  9
         ifa(1, 3) = 10 
         ifa(2, 3) =  5 
         ifa(3, 3) =  3
         ifa(1, 4) =  5 
         ifa(2, 4) = 10 
         ifa(3, 4) =  9
         ifa(1, 5) =  4 
         ifa(2, 5) = 10 
         ifa(3, 5) =  3
         ifa(1, 6) = 10 
         ifa(2, 6) =  4 
         ifa(3, 6) =  7
         ifa(1, 7) =  2 
         ifa(2, 7) = 11 
         ifa(3, 7) =  1
         ifa(1, 8) =  9 
         ifa(2, 8) =  2 
         ifa(3, 8) =  1
         ifa(1, 9) =  7 
         ifa(2, 9) =  2 
         ifa(3, 9) =  9
         ifa(1,10) = 12 
         ifa(2,10) =  4 
         ifa(3,10) =  3
         ifa(1,11) =  6 
         ifa(2,11) =  5 
         ifa(3,11) =  1
         ifa(1,12) = 11 
         ifa(2,12) =  6 
         ifa(3,12) =  1
         ifa(1,13) = 12 
         ifa(2,13) =  6 
         ifa(3,13) = 11
         ifa(1,14) =  5 
         ifa(2,14) =  6 
         ifa(3,14) =  3
         ifa(1,15) =  6 
         ifa(2,15) = 12 
         ifa(3,15) =  3
         ifa(1,16) =  8 
         ifa(2,16) =  2 
         ifa(3,16) =  7
         ifa(1,17) =  2 
         ifa(2,17) =  8 
         ifa(3,17) = 11
         ifa(1,18) =  8 
         ifa(2,18) = 12 
         ifa(3,18) = 11
         ifa(1,19) =  4 
         ifa(2,19) =  8 
         ifa(3,19) =  7
         ifa(1,20) = 12 
         ifa(2,20) =  8 
         ifa(3,20) =  4
c
      else if (ipol .eq. 2) then
c
c........cube:
c
         rad0 = sqrt(3d0)
         nv = 8
         x(1) = +1
         y(1) = +1
         z(1) = +1
         x(2) = +1
         y(2) = +1
         z(2) = -1
         x(3) = +1
         y(3) = -1
         z(3) = +1
         x(4) = +1
         y(4) = -1
         z(4) = -1
         x(5) = -1
         y(5) = +1
         z(5) = +1
         x(6) = -1
         y(6) = +1
         z(6) = -1
         x(7) = -1
         y(7) = -1
         z(7) = +1
         x(8) = -1
         y(8) = -1
         z(8) = -1
         nf = 6
         nvf = 4
         ifa(1,1) = 1
         ifa(2,1) = 2
         ifa(3,1) = 4
         ifa(4,1) = 3
         ifa(1,2) = 5
         ifa(2,2) = 6
         ifa(3,2) = 2
         ifa(4,2) = 1
         ifa(1,3) = 7
         ifa(2,3) = 8
         ifa(3,3) = 6
         ifa(4,3) = 5
         ifa(1,4) = 3
         ifa(2,4) = 4
         ifa(3,4) = 8
         ifa(4,4) = 7
         ifa(1,5) = 5
         ifa(2,5) = 1
         ifa(3,5) = 3
         ifa(4,5) = 7
         ifa(1,6) = 2
         ifa(2,6) = 6
         ifa(3,6) = 8
         ifa(4,6) = 4
      else if (ipol .eq. 3) then
c
c........octahedron:
c
         rad0 = 1d0
         nv = 6
         x(1) = +1
         y(1) =  0
         z(1) =  0
         x(2) = -1
         y(2) =  0
         z(2) =  0
         x(3) =  0
         y(3) = +1
         z(3) =  0
         x(4) =  0
         y(4) = -1
         z(4) =  0
         x(5) =  0
         y(5) =  0
         z(5) = +1
         x(6) =  0
         y(6) =  0
         z(6) = -1
         nf = 8
         nvf = 3
         ifa(1,1) = 3
         ifa(2,1) = 5
         ifa(3,1) = 2
         ifa(1,2) = 5
         ifa(2,2) = 3
         ifa(3,2) = 1
         ifa(1,3) = 6
         ifa(2,3) = 3
         ifa(3,3) = 2
         ifa(1,4) = 3
         ifa(2,4) = 6
         ifa(3,4) = 1
         ifa(1,5) = 5
         ifa(2,5) = 4
         ifa(3,5) = 2
         ifa(1,6) = 4
         ifa(2,6) = 5
         ifa(3,6) = 1
         ifa(1,7) = 4
         ifa(2,7) = 6
         ifa(3,7) = 2
         ifa(1,8) = 6
         ifa(2,8) = 4
         ifa(3,8) = 1
      else if (ipol .eq. 4) then
c
c........tetrahedron:
c
         rad0 = sqrt(3d0)
         nv = 4
         x(1) = +1
         y(1) = +1
         z(1) = +1
         x(2) = -1
         y(2) = -1
         z(2) = +1
         x(3) = +1
         y(3) = -1
         z(3) = -1
         x(4) = -1
         y(4) = +1
         z(4) = -1
         nf = 4
         nvf = 3
         ifa(1,1) = 1
         ifa(2,1) = 3
         ifa(3,1) = 2
         ifa(1,2) = 4
         ifa(2,2) = 1
         ifa(3,2) = 2
         ifa(1,3) = 3
         ifa(2,3) = 4
         ifa(3,3) = 2
         ifa(1,4) = 4
         ifa(2,4) = 3
         ifa(3,4) = 1
c
      endif
c
      do i = 1, nv
         x(i) = x0 + x(i) * radius / rad0
         y(i) = y0 + y(i) * radius / rad0
         z(i) = z0 + z(i) * radius / rad0
      enddo
c
      end
