c
c-----------------------------------------------------------------------
c
      subroutine writeoff (offfile, title, uout)
c
c.....writeoff - write the polyhedron description as an OFF file.
c     Either the tessel database or the molecular graphics database
c     can be written at once, but no both simultaneously. This should
c     not be a serious drawback when the files are viewed with Geomview,
c     however, as two different files can be viewed together.
c
      include           'implicit.inc'
      include           'error.inc'
      include           'polyhedron.inc'
      include           'molge.inc'
      include           'strtools.inc'
c
      character*(*)     offfile, title
      integer           uout
c
      integer           luo, i, j, k, i1, i2, j1, nnn
      real*8            fmin, fmax, fdel, ff, cred, cgrn, cblu, cgry
c
      integer           locMV, locMF, locMVF
      parameter         (locMV = 16, locMF = 20, locMVF = GMSIDE)
      real*8            locx(locMV), locy(locMV), locz(locMV)
      integer           locnv, locnf, locnvf, locifa(locMVF,locMF)
      integer           locnvf1(locMVF)
c
      integer           oldgnpt, oldgnpol, oldgnplh
      character*(20)    formato
c
      integer           leng, fopen
c
c.....Open the output file:
c
      if (fopen (luo, offfile, iowrite) .eq. iowrite) then
         call error ('writeoff', 'I/O error when opening off file',
     &               warning)
         return
      endif
c
c.....Check which database to write:
c
      if (nvert .gt. 0) then
         goto 1
      else if (gnpt .gt. 0) then
         goto 2
      else
         goto 3
      endif
c
c.......................................................................
c..... WRITE THE TESSEL DATABASE FIRST .................................
c.......................................................................
c
 1    continue
      if (color_mode .eq. color_none) then
c
c........Write vertices if no color is used:
c
         write (luo,100) title(1:leng(title)), 'OFF'
         write (luo,105) nvert, nface(isw1), nvert+nface(isw1)-2
         do j = 1, nvert
            write (luo,110) xvert(j), yvert(j), zvert(j)
         enddo
c
      else if (color_mode .eq. color_binary) then
c
c........Write the vertices using red/write color depending on the
c        sign of the function value at the point:
c
         write (luo,100) title(1:leng(title)), 'COFF'
         write (luo,105) nvert, nface(isw1), nvert+nface(isw1)-2
         do j = 1, nvert
            if (fun_value(j) .gt. 0) then
               write (luo,110) xvert(j), yvert(j), zvert(j),
     &                         1d0, 0d0, 0d0, 0.5d0
            else if (fun_value(j) .lt. 0) then
               write (luo,110) xvert(j), yvert(j), zvert(j),
     &                         0d0, 1d0, 0d0, 0.5d0
            else
               write (luo,110) xvert(j), yvert(j), zvert(j),
     &                         1d0, 1d0, 0d0, 0.5d0
            endif
         enddo
c
      else if (color_mode .eq. color_hue) then
c
c........Use the function value to define the vertex HUE.
c........(Saturation & value are 1 by default).
c
         write (luo,100) title(1:leng(title)), 'COFF'
         write (luo,105) nvert, nface(isw1), nvert+nface(isw1)-2
         do j = 1, nvert
            ff = fun_value(j)
            ff = abs(ff)
            call hsv2rgb (ff, 1d0, 1d0, cred, cgrn, cblu)
            write (luo,110) xvert(j), yvert(j), zvert(j),
     &                      cred, cgrn, cblu, 0.5d0
         enddo
c
      else
c
c........Use a grey scale or a rainbow full color scale.
c........Determine first minimum and maximum values of the function:
c
         if (useminmax) then
            fmin = fun_min
            fmax = fun_max
         else
            fmin = fun_value(1)
            fmax = fun_value(1)
            do j = 2, nvert
               if (fun_value(j) .lt. fmin) fmin = fun_value(j)
               if (fun_value(j) .gt. fmax) fmax = fun_value(j)
            enddo
         endif
         fdel = 2d0 / (fmax-fmin)
         write (luo,100) title(1:leng(title)), 'COFF'
         write (luo,105) nvert, nface(isw1), nvert+nface(isw1)-2
         if (color_mode .eq. color_grey) then 
            do j = 1, nvert
               ff = (fun_value(j)-fmin)*fdel-1d0
               call grayscale (ff, cgry)
               write (luo,110) xvert(j), yvert(j), zvert(j),
     &                         cgry, cgry, cgry, 0.5d0
            enddo
         else if (color_mode .eq. color_full) then 
            do j = 1, nvert
               ff = (fun_value(j)-fmin)*fdel-1d0
               call colorscale (ff, cred, cgrn, cblu, 1d0, 3d0, 1d0)
               write (luo,110) xvert(j), yvert(j), zvert(j),
     &                         cred, cgrn, cblu, 0.5d0
            enddo
         else
            call error ('writeoff', 'Unknown color mode', faterr)
         endif
c
      endif
c
c.....Write now the faces of the polyhedron:
c
      do j = 1, nface(isw1)
         write (luo,115) nvf, (iface(i,j,isw1)-1, i = 1, nvf)
      enddo
c
      write (uout,200) offfile(1:leng(offfile)), nvert, nvf, nface(isw1)
      goto 3
c
c.......................................................................
c..... WRITE THE GRAPHICAL ELEMENTS DATABASE ...........................
c.......................................................................
c
 2    continue
c
c.....Balls, sticks and cones will be translated into polyhedra:
c.....Convert balls ...
c
      oldgnpt = gnpt
      oldgnpol = gnpol
      oldgnplh = gnplh
      do i = 1, gnball
         i1 = gballct(i)
         call sphere (gxpt(i1), gypt(i1), gzpt(i1), gballrad(i), 1,
     &                locx, locy, locz, locnv,
     &                locnf, locnvf, locMVF, locifa)
         if (gnpt+locnv .gt. GMPT) call error ('writeoff',
     &      'Increase GMPT to allow conversion of balls', faterr)
         if (gnplh+1 .gt. GMPLH) call error ('writeoff',
     &      'Increase GMPLH to allow conversion of balls', faterr)
         if (gnpol+locnf .gt. GMPOL) call error ('writeoff',
     &      'Increase GMPOL to allow conversion of balls', faterr)
         gnplh = gnplh + 1
         gnvert(gnplh) = locnv
         gnface(gnplh) = locnf
         gplhrgbf(1,gnplh) = gballrgbf(1,i)
         gplhrgbf(2,gnplh) = gballrgbf(2,i)
         gplhrgbf(3,gnplh) = gballrgbf(3,i)
         gplhrgbf(4,gnplh) = gballrgbf(4,i)
         do j = 1, locnf
            gnpol = gnpol + 1
            giface(j,gnplh) = gnpol
            gnside(gnpol) = locnvf
            do k = 1, locnvf
               giside(k,gnpol) = locifa(k,j) + gnpt
            enddo
         enddo
         do j = 1, locnv
            gnpt = gnpt + 1
            givert(j,gnplh) = gnpt
            gxpt(gnpt) = locx(j)
            gypt(gnpt) = locy(j)
            gzpt(gnpt) = locz(j)
         enddo
      enddo
c
c.....convert sticks ...
c
      do i = 1, gnstk
         i1 = gstkpt(1,i)
         i2 = gstkpt(2,i)
         call cylinder (
     &        gxpt(i1), gypt(i1), gzpt(i1),
     &        gxpt(i2), gypt(i2), gzpt(i2), gstkrad(i), 8,
     &                locx, locy, locz, locnv,
     &                locnf, locnvf, locMVF, locifa)
         if (gnpt+locnv .gt. GMPT) call error ('writeoff',
     &      'Increase GMPT to allow conversion of sticks', faterr)
         if (gnplh+1 .gt. GMPLH) call error ('writeoff',
     &      'Increase GMPLH to allow conversion of sticks', faterr)
         if (gnpol+locnf .gt. GMPOL) call error ('writeoff',
     &      'Increase GMPOL to allow conversion of sticks', faterr)
         gnplh = gnplh + 1
         gnvert(gnplh) = locnv
         gnface(gnplh) = locnf
         gplhrgbf(1,gnplh) = gstkrgbf(1,i)
         gplhrgbf(2,gnplh) = gstkrgbf(2,i)
         gplhrgbf(3,gnplh) = gstkrgbf(3,i)
         gplhrgbf(4,gnplh) = gstkrgbf(4,i)
         do j = 1, locnf
            gnpol = gnpol + 1
            giface(j,gnplh) = gnpol
            gnside(gnpol) = locnvf
            do k = 1, locnvf
               giside(k,gnpol) = locifa(k,j) + gnpt
            enddo
         enddo
         do j = 1, locnv
            gnpt = gnpt + 1
            givert(j,gnplh) = gnpt
            gxpt(gnpt) = locx(j)
            gypt(gnpt) = locy(j)
            gzpt(gnpt) = locz(j)
         enddo
      enddo
c
c.....convert cones ...
c
      do i = 1, gncone
         i1 = gcnpt(1,i)
         i2 = gcnpt(2,i)
         call cone (
     &        gxpt(i1), gypt(i1), gzpt(i1),
     &        gxpt(i2), gypt(i2), gzpt(i2), gcnrad(i), 8,
     &                locx, locy, locz, locnv,
     &                locnf, locnvf1, locMVF, locifa)
         if (gnpt+locnv .gt. GMPT) call error ('writeoff',
     &      'Increase GMPT to allow conversion of cones', faterr)
         if (gnplh+1 .gt. GMPLH) call error ('writeoff',
     &      'Increase GMPLH to allow conversion of cones', faterr)
         if (gnpol+locnf .gt. GMPOL) call error ('writeoff',
     &      'Increase GMPOL to allow conversion of cones', faterr)
         gnplh = gnplh + 1
         gnvert(gnplh) = locnv
         gnface(gnplh) = locnf
         gplhrgbf(1,gnplh) = gcnrgbf(1,i)
         gplhrgbf(2,gnplh) = gcnrgbf(2,i)
         gplhrgbf(3,gnplh) = gcnrgbf(3,i)
         gplhrgbf(4,gnplh) = gcnrgbf(4,i)
         do j = 1, locnf
            gnpol = gnpol + 1
            giface(j,gnplh) = gnpol
            gnside(gnpol) = locnvf1(j)
            do k = 1, locnvf1(j)
               giside(k,gnpol) = locifa(k,j) + gnpt
            enddo
         enddo
         do j = 1, locnv
            gnpt = gnpt + 1
            givert(j,gnplh) = gnpt
            gxpt(gnpt) = locx(j)
            gypt(gnpt) = locy(j)
            gzpt(gnpt) = locz(j)
         enddo
      enddo
c
c.....Write out the polyhedra:
c
      write (luo,100) title(1:leng(title)), 'OFF'
      nnn = 0
      do i = 1, gnplh
         nnn = nnn + gnface(i)
      enddo
*     write (luo,105) gnpt, gnpol, 0
      write (luo,105) gnpt, nnn, 0
      do j = 1, gnpt
         write (luo,110) gxpt(j), gypt(j), gzpt(j)
      enddo
*     do j = 1, gnpol
*        write (luo,115) gnside(j), (giside(i,j)-1, i = 1, gnside(j))
*     enddo
      do i = 1, gnplh
         do j = 1, gnface(i)
            j1 = giface(j,i)
            write (formato,116) gnside(j1)+1
            write (luo,formato) gnside(j1),
     &            (giside(k,j1)-1, k = 1, gnside(j1)),
     &            (gplhrgbf(k,i), k = 1, 4)
         enddo
      enddo
c
      write (uout,205) offfile(1:leng(offfile)), gnpt, gnplh, gnpol
c
c.....Restore the graphical elements database to its original state:
c
      gnpt = oldgnpt
      gnpol = oldgnpol
      gnplh = oldgnplh
c
c.......................................................................
c..... END THE ROUTINE .................................................
c.......................................................................

 3    continue
      call fclose (luo)
c
 100  format (
     & '# Polyhedron created by TESSEL:'/
     & '# (c) 1996 V. Lua~na, Departamento de Quimica Fisica'/
     & '#     Universidad de Oviedo, 33006-Oviedo, Spain'/
     & '#     victor@hobbit.quimica.uniovi.es'/
     & '#'/
     & '# ', a/
     & '#'/
     & a)
 105  format (3i7)
 110  format (3f15.9, 3f6.3, f4.1)
 115  format (10i7)
 116  format ('(',i2,'i7, 3f6.3, f4.1)')
c
 200  format (/
     & 1x, 'WRITEOFF: writing a description of the object in the OFF'/
     & 1x, 'format to be viewed and manipulated with Geomview.'//
     & 1x, 'Name of the OFF file:        ', a/
     & 1x, 'Number of vertices created:  ', i6/
     & 1x, 'Number of vertices per face: ', i6/
     & 1x, 'Number of faces:             ', i6)
 205  format (/
     & 1x, 'WRITEOFF: writing the molecular graphics database in the '/
     & 1x, 'OFF format to be viewed and manipulated with Geomview.'//
     & 1x, 'Name of the OFF file:        ', a/
     & 1x, 'Number of vertices created:  ', i6/
     & 1x, 'Number of polyhedra created: ', i6/
     & 1x, 'Number of polygons (faces):  ', i6)
c
      end
