c
c-----------------------------------------------------------------------
c
      subroutine writevrml1 (vfile, title, uout)
c
c.....writevrml1 - write the polyhedron description as an VRML 1.0 file.
c
      include           'implicit.inc'
      include           'error.inc'
      include           'consts.inc'
      include           'polyhedron.inc'
      include           'molge.inc'
      include           'strtools.inc'
c
      character*(*)     vfile, title
      integer           uout
c
      integer           luo, i, j, k, i1, i2, i3, isum, j1
      real*8            rr, theta, phi
      real*8            fmin, fmax, fdel, ff
      real*8            cred, cgrn, cblu
c
      integer           leng, fopen
c
c.....Open the output file:
c
      if (fopen (luo, vfile, iowrite) .eq. iowrite) then
         call error ('writevrml1', 'I/O error when opening VRML file',
     &               warning)
         return
      endif
c
c.....Write first the file header:
c
      write (luo,100) title(1:leng(title))
 100  format (
     & '#VRML V1.0 ascii'/
     & 'Separator {'//
     & '   Info { string "Model created by TESSEL" }'/
     & '   Info { string "(c) 1996 V. Lua~na" }'/
     & '   Info { string "    Departamento de Quimica Fisica" }'/
     & '   Info { string "    Universidad de Oviedo" }'/
     & '   Info { string "    33006-Oviedo, Spain" }'/
     & '   Info { string "    victor@carbono.quimica.uniovi.es" }'/
     & '   Info { string "', a, '" }'//
     & '   DEF Def_Color Material { diffuseColor 1 0 0 }'/
     & '   DEF Red   Material { diffuseColor 1 0 0 }'/
     & '   DEF Green Material { diffuseColor 0 1 0 }'/
     & '   DEF Blue  Material { diffuseColor 0 0 1 }'/
     & )
c
c.......................................................................
c..... WRITE THE TESSEL DATABASE FIRST .................................
c.......................................................................
c
      if (nvert .le. 0) goto 2
 1    continue
      write (luo,110)
c
      if (color_mode .eq. color_none) then
c
c........Use a solid color for all:
c
         write (luo,120)
         do j = 1, nvert-1
            write (luo,122) xvert(j), yvert(j), zvert(j)
         enddo
         write (luo,123) xvert(nvert), yvert(nvert), zvert(nvert)
         write (luo,126)
         do j = 1, nface(isw1)
            write (luo,128) (iface(i,j,isw1)-1, i = 1, nvf), -1
         enddo
         write (luo,130)
c
      else if (color_mode .eq. color_binary) then
c
c........Color each face as Red/Green according to the value of a
c        function on the vertices:
c
         write (luo,140)
         do j = 1, nvert-1
            write (luo,142) xvert(j), yvert(j), zvert(j)
         enddo
         write (luo,143) xvert(nvert), yvert(nvert), zvert(nvert)
         write (luo,144)
         do j = 1, nface(isw1)
            isum = 0
            do i = 1, nvf
               if (fun_value(iface(i,j,isw1)) .lt. 0d0) isum = isum + 1
            enddo
            if (isum .gt. 1) then
               write (luo,145) 0, 1, 0
            else
               write (luo,145) 1, 0, 0
            endif
         enddo
         write (luo,148)
         write (luo,150)
         do j = 1, nface(isw1)
            write (luo,152) (iface(i,j,isw1)-1, i = 1, nvf), -1
         enddo
         write (luo,154)
c
      else if (color_mode .eq. color_hue) then
c
c........Use the function value as the HUE of the vertices.
c        Saturation and value will be 1 by default.
c
         write (luo,140)
         do j = 1, nvert-1
            write (luo,142) xvert(j), yvert(j), zvert(j)
         enddo
         write (luo,143) xvert(nvert), yvert(nvert), zvert(nvert)
         write (luo,144)
         do i = 1, nface(isw1)
            i1 = iface(1,i,isw1)
            i2 = iface(2,i,isw1)
            i3 = iface(3,i,isw1)
            ff = (fun_value(i1)+fun_value(i2)+fun_value(i3)) / 3d0
            ff = abs(ff)
            call hsv2rgb (ff, 1d0, 1d0, cred, cgrn, cblu)
            write (luo,146) cred, cgrn, cblu
         enddo
         write (luo,148)
         write (luo,150)
         do j = 1, nface(isw1)
            write (luo,152) (iface(i,j,isw1)-1, i = 1, nvf), -1
         enddo
         write (luo,154)
c
      else
c
c........Use grey or rainbow scale colors:
c........Determine first minimum and maximum function values:
c
         if (useminmax) then
            fmin = fun_min
            fmax = fun_max
         else
            fmin = fun_value(1)
            fmax = fun_value(1)
            do j = 2, nvert
               if (fun_value(j) .lt. fmin) fmin = fun_value(j)
               if (fun_value(j) .gt. fmax) fmax = fun_value(j)
            enddo
         endif
         fdel = 2d0 / (fmax-fmin)
         write (luo,140)
         do j = 1, nvert-1
            write (luo,142) xvert(j), yvert(j), zvert(j)
         enddo
         write (luo,143) xvert(nvert), yvert(nvert), zvert(nvert)
         write (luo,144)
         do i = 1, nface(isw1)
            i1 = iface(1,i,isw1)
            i2 = iface(2,i,isw1)
            i3 = iface(3,i,isw1)
            ff = (fun_value(i1)+fun_value(i2)+fun_value(i3)) / 3d0
            ff = (ff-fmin)*fdel-1d0
            if (color_mode .eq. color_grey) then
               call grayscale (ff, cred)
               cgrn = cred
               cblu = cred
            else
               call colorscale (ff, cred, cgrn, cblu, 1d0, 3d0, 1d0)
            endif
            write (luo,146) cred, cgrn, cblu
         enddo
         write (luo,148)
         write (luo,150)
         do j = 1, nface(isw1)
            write (luo,152) (iface(i,j,isw1)-1, i = 1, nvf), -1
         enddo
         write (luo,154)
c
      endif
c
      write (luo,199)
c
 110  format (
     & '   Separator { #Tessel database')
c
 120  format (
     & '      USE Def_Color'/
     & '      Coordinate3 { point [')
 122  format (9x, 3f15.9, ',')
 123  format (9x, 3f15.9 /
     & '         ] }')
 126  format (
     & '      IndexedFaceSet { coordIndex [')
 128  format (9x, 40(i4, ','))
 130  format (
     & '         ] }')
c
 140  format (
     & '      Coordinate3 { point [')
 142  format (9x, 3f15.9, ',')
 143  format (9x, 3f15.9 /
     & '         ] }')
 144  format (
     & '      Material { diffuseColor [')
 145  format (9x, 3i4, ',')
 146  format (9x, 3f8.3, ',')
 148  format (
     & '         ] }')
 150  format (
     & '      MaterialBinding { value PER_FACE }'/
     & '      IndexedFaceSet {'/
     & '         coordIndex [')
 152  format (12x, 10(i4, ','))
 154  format (
     & '            ]'/
     & '         }')
c
 199  format (
     & '      }'/)
c
c.......................................................................
c..... WRITE THE GRAPHICAL ELEMENTS DATABASE ...........................
c.......................................................................
c
 2    continue
      if (gnpt .le. 0) goto 3
c
c.....Write out balls:
c
      do i = 1, gnball
         i1 = gballct(i)
         write (luo,200) i,
     &         gballrgbf(1,i), gballrgbf(2,i), gballrgbf(3,i),
     &         gballrgbf(4,i),
     &         gxpt(i1), gypt(i1), gzpt(i1),
     &         gballrad(i)
      enddo
 200  format (
     & '   Separator { #Ball number ', i4/
     & '      Material {'/
     & '         diffuseColor ', 3f10.3/
     & '         transparency ', f10.3/
     & '         }'/
     & '      Transform { translation ', 3f12.4, ' }'/
     & '      Sphere { radius ', f12.4, ' }'/
     & '      }'/ )
c
c.....Write out sticks:
c     (VRML 1.0 sticks point along the Y axis and are centered on 0,0,0)
c
      do i = 1, gnstk
         i1 = gstkpt(1,i)
         i2 = gstkpt(2,i)
c
c........spherical coordinates of the top relative to the bottom of the
c        cylinder:
c
         rr = sqrt( (gxpt(i2)-gxpt(i1))**2 + (gypt(i2)-gypt(i1))**2 +
     &              (gzpt(i2)-gzpt(i1))**2 )
         if (rr .ge. EPS) then
            phi = atan2( gypt(i2)-gypt(i1) , gxpt(i2)-gxpt(i1) )
            theta = acos( (gzpt(i2)-gzpt(i1)) / rr )
            write (luo,210) i,
     &         gstkrgbf(1,i), gstkrgbf(2,i), gstkrgbf(3,i),
     &         gstkrgbf(4,i), 
     &         gxpt(i1), gypt(i1), gzpt(i1),
     &         phi - 0.5d0*pi,
     &         0.5d0*pi - theta,
     &         0.5d0 * rr,
     &         gstkrad(i), rr
         endif
      enddo
 210  format (
     & '   Separator { #Stick number ', i4/
     & '      Material {'/
     & '         diffuseColor ', 3f10.3/
     & '         transparency ', f10.3/
     & '         }'/
     & '      Translation { translation ', 3f12.4, '}'/
     & '      Rotation { rotation 0 0 1 ', f12.4, '}'/
     & '      Rotation { rotation 1 0 0 ', f12.4, '}'/
     & '      Translation { translation 0 ', f12.4, ' 0 }'/
     & '      Cylinder {'/
     & '         parts ALL'/
     & '         radius ', f12.4/
     & '         height ', f12.4/
     & '         }'/
     & '      }'/ )
c
c.....Write out cones:
c     (VRML 1.0 cones point along the Y axis and are centered on 0,0,0)
c
      do i = 1, gncone
         i1 = gcnpt(1,i)
         i2 = gcnpt(2,i)
c
c........spherical coordinates of the apex relative to the base of the
c        cone:
c
         rr = sqrt( (gxpt(i2)-gxpt(i1))**2 + (gypt(i2)-gypt(i1))**2 +
     &              (gzpt(i2)-gzpt(i1))**2 )
         if (rr .ge. EPS) then
            phi = atan2( gypt(i2)-gypt(i1) , gxpt(i2)-gxpt(i1) )
            theta = acos( (gzpt(i2)-gzpt(i1)) / rr )
            write (luo,220) i,
     &         gcnrgbf(1,i), gcnrgbf(2,i), gcnrgbf(3,i),
     &         gcnrgbf(4,i), 
     &         gxpt(i1), gypt(i1), gzpt(i1),
     &         phi - 0.5d0*pi,
     &         0.5d0*pi - theta,
     &         0.5d0 * rr,
     &         gcnrad(i), rr
         endif
      enddo
 220  format (
     & '   Separator { #Cone number ', i4/
     & '      Material {'/
     & '         diffuseColor ', 3f10.3/
     & '         transparency ', f10.3/
     & '         }'/
     & '      Translation { translation ', 3f12.4, '}'/
     & '      Rotation { rotation 0 0 1 ', f12.4, '}'/
     & '      Rotation { rotation 1 0 0 ', f12.4, '}'/
     & '      Translation { translation 0 ', f12.4, ' 0 }'/
     & '      Cone {'/
     & '         parts ALL'/
     & '         bottomradius ', f12.4/
     & '         height ', f12.4/
     & '         }'/
     & '      }'/ )
c
c.....Write out the polyhedra:
c
      if (gnplh .gt. 0) then
         write (luo,230)
         do j = 1, gnpt-1
            write (luo,232) gxpt(j), gypt(j), gzpt(j)
         enddo
         write (luo,233) gxpt(gnpt), gypt(gnpt), gzpt(gnpt)
         write (luo,234)
         do i = 1, gnplh
            write (luo,236) i, (gplhrgbf(k,i), k = 1, 4)
            do j = 1, gnface(i)
               j1 = giface(j,i)
               write (luo,238) (giside(k,j1)-1, k = 1, gnside(j1)), -1
            enddo
            write (luo,240)
         enddo
         write (luo,242)
      endif
c
 230  format (
     & '   Separator { #Molecular polyhedra'/
     & '      Coordinate3 { point [' )
 232  format (
     & '       ', 3f12.5, ',' )
 233  format (
     & '       ', 3f12.5 )
 234  format (
     & '         ] }' )
 236  format (
     & '      #Polyhedron number ', i4/
     & '      Material {'/
     & '         diffuseColor ', 3f10.3/
     & '         transparency ', f10.3/
     & '         }'/
     & '      IndexedFaceSet { coordIndex [' )
 238  format (
     & '         ', 40(i4, ',') )
 240  format (
     & '         ] }' )
 242  format (
     & '      }' / )
c
c.......................................................................
c..... END THE ROUTINE .................................................
c.......................................................................
c
 3    continue
c
c.....Finally write the file end:
c
      write (luo,300)
 300  format (
     & '   }' )
      call fclose (luo)
c
      write (uout,900) vfile(1:leng(vfile)), nvert, nvf, nface(isw1),
     &                 gnball, gnstk, gncone, gnplh
c
 900  format (/
     & 1x, 'WRITEVRML1: writing a description of the object in the'/
     & 1x, 'VRML format to be plot and manipulated with VRweb'/
     & 1x, 'or similar virtual reality visualizer'//
     & 1x, 'Name of the VRML file:       ', a/
     & 1x, '-- POLYHEDRAL OBJECT --'/
     & 1x, 'Number of vertices created:  ', i6/
     & 1x, 'Number of vertices per face: ', i6/
     & 1x, 'Number of faces:             ', i6/
     & 1x, '-- MOLECULAR OBJECT --'/
     & 1x, 'Number of balls:     ', i6/
     & 1x, 'Number of sticks:    ', i6/
     & 1x, 'Number of cones:     ', i6/
     & 1x, 'Number of polyhedra: ', i6)
c
      end
